/**
 * Date: 3/08/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Parent File Names: EditExpense.java, AddExpense.java
 * Referenced Files: 
 * 
 * Class Version: 1.0
 * Change Log:
 * 03/08/2011	Class Created
 * 
 * Notes: 
 * This class is used to store default categories and any user categories created. 
 * It can write itself to a file so the created list will persist.
 * 
 * Things TODO:
 * 1. Review Code and clean up and streamline as necessary
 */
package com.garryburns.ExpenseIT;

import java.io.*;
import javax.swing.DefaultListModel;


public class CategoriesList extends DefaultListModel implements Serializable {
 private static final long serialVersionUID = 1L;
 private String[] demoCatNames = {"Office Supplies", "Dining", "Entertainment", 
		 							 "R&D", "Education", "Groceries"};
 private Category[] demoCategories = new Category[demoCatNames.length]; 	
 
 //Constructor
 CategoriesList(){
	super();

	//Add the default Categories
	for(int i = 0; i < demoCatNames.length; ++i) {
		demoCategories[i] = new Category(demoCatNames[i], false);
		addElement(demoCategories[i]);
	}
	
	//Set the first 4 demo categories to be default categories
	for(int i = 0; i < 4; ++i) {
		demoCategories[i].setIsDefault(true);
	}
	saveList(this);
 }
 
 void addCategory(String inName, boolean inDefault) {
	 Category cat = new Category(inName, inDefault);
	 addElement(cat);
	 saveList(this);
 }
 void saveList(CategoriesList list) {
	 try {
		 FileOutputStream fo = new FileOutputStream("ExpenseIT.cats");
		 ObjectOutputStream oo = new ObjectOutputStream(fo);
		 oo.writeObject(list);
		 oo.close();
	 } catch (IOException e) {
		 System.out.println("Error - " + e.toString());
	 }
 }
 CategoriesList getList() {
	 CategoriesList list;
	 try {
		 FileInputStream fi = new FileInputStream("ExpenseIT.cats");
		 ObjectInputStream oi = new ObjectInputStream(fi);
		 list = (CategoriesList) oi.readObject();
		 // TODO add some testing to ensure list is viable
		 oi.close();
		 return list;
	 } catch (Exception e) {
		 System.out.println("Error - " + e.toString());
		 return null;
	 }
 }
}

class Category implements Serializable {
 private static final long serialVersionUID = 1L;
 private String name;
 private boolean isDefault;
 
 Category(String inName, boolean inIsDefault) {
	 name = inName;
	 isDefault = inIsDefault;	 
 }
 
 void setName(String inName){
	 name = inName;  
 }
 void setIsDefault(boolean inIsDefault) {
	 isDefault = inIsDefault;
 }
 String getName() {
	 return name;
 }
 boolean getIsDefault() {
	 return isDefault;
 }
}
