/**
 * Date: 3/14/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Parent File Names: SettingsPane.java
 * Referenced Files: 
 * 			CategoryList.java
 * 
 * Class Version: 1
 * Change Log:
 * 03/14/2011	Class Created
 * 
 * Notes: 
 * This class creates the Pane which includes a table where the user can edit, add, delete
 * 	the program's categories.
 * 
 * Things TODO:
 * 1. Review Code and clean up and streamline as necessary
 */
package com.garryburns.ExpenseIT;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

@SuppressWarnings("serial")
class EditLocsPane extends JPanel implements ActionListener {
 private static JButton jbDelete, jbClearAll;
 static JButton jbBack;
 private static int indexSelected;
 LocationsTableModel tableData;
 JTable table;
 
 EditLocsPane() {
	super();
	
	jbDelete = new JButton("Delete");
	jbBack = new JButton("Back");
	jbClearAll = new JButton("Clear All");
	
	jbDelete.addActionListener(this);
	jbClearAll.addActionListener(this);
    
	tableData = new LocationsTableModel();
	table = new JTable();
	table.setModel(tableData);
	table.setPreferredScrollableViewportSize(new Dimension(300, 300));
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	table.setFillsViewportHeight(true);
	JScrollPane scroller = new JScrollPane(table);
	
	add(scroller);
	add(jbDelete);
	add(jbBack);
	add(jbClearAll);
 }

 @Override
 public void actionPerformed(ActionEvent evt) {
	JButton jbSource = (JButton) evt.getSource();
	
	if(jbSource == jbDelete) {
		deleteLoc();
		enableDelete();
	}
	if(jbSource == jbClearAll) {
		resetCategories();
	}
 }
 private void resetCategories() {
	ExpenseIT.settings.clearLocations();
	tableData = new LocationsTableModel();
	table.setModel(tableData);
	table.repaint();
	enableDelete();
	repaint();
}  
   
 private void deleteLoc() {
	 //Delete the selected category then show the new list
	 indexSelected = table.getSelectedRow();
	 
	 //Otherwise continue with removal
	 ExpenseIT.settings.locations.remove(indexSelected);
	 Settings.saveSettings(ExpenseIT.settings);

     setRow(indexSelected);
     table.repaint();
	 repaint();
	 
 }
 private void setRow(int row) {
	if(row > 0 && row < tableData.getRowCount())
		table.setRowSelectionInterval(row, row);
	else 
		table.setRowSelectionInterval(0, 0);
	
 }

 void enableDelete() {
	if(tableData.getRowCount() < 1)				//Disable Delete if there's nothing to delete
		jbDelete.setEnabled(false);
	else
		jbDelete.setEnabled(true);
 }
}

@SuppressWarnings("serial")
class LocationsTableModel extends AbstractTableModel {
 private String[] columnNames = {"Location"};
 Vector<String> data;
 
 LocationsTableModel() {
	 data = ExpenseIT.settings.locations;
 }

 @Override
 public int getColumnCount() {
	return columnNames.length;
 }

 @Override
 public int getRowCount() {
	return data==null ? 0 : data.size();
 }

 @Override
 public String getColumnName(int col) {
	return columnNames[col];
 }
 @Override
 public Object getValueAt(int row, int col) {
    if (row < 0 || row>=getRowCount())
        return "";
    return data.get(row);
 }
 @Override
 public Class<?> getColumnClass(int c) {
	return getValueAt(0, c).getClass();
 }
 @Override
 public boolean isCellEditable(int row, int col) {
	return true;
 }
 @Override
 public void setValueAt(Object value, int nRow, int nCol) {
    if (nRow < 0 || nRow>=getRowCount())
      return;
    ExpenseIT.settings.locations.set(nRow, value.toString());
    Settings.saveSettings(ExpenseIT.settings);
 }
}