/**
 * Date: 3/03/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Parent File Names: N/A
 * Subfile Names: 
 * 			AddExpense.java
 * 			Calculator.java
 * 			Expense.java
 * 			ExpenseList.java
 * 
 * Class Version: 0.2.3
 * Change Log:
 * 03/09/2011	Implemented CategoriesList class - CategoriesList.java 
 * 03/08/2011	Added Undo Button - jbUndo; Added labels to edit page
 * 				Made changes to AddExpense.java, ExpenseList.java
 * 03/07/2011	Consolidated EditPane and Home Frame into this class
 * 03/03/2011	Class Created
 * 
 * 
 * Notes: 
 * This is the main class for the program ExpenseIT
 * it makes the expense list pane the home page of the whole program.
 * 
 * Things TODO:
 * 1. Clean up UI
 * 2. Add settings control and functions
 * 3. Add some testing controls to the loadList() method to ensure the 
 * 		file loaded is acceptable
 * 4. Add an about page - probably put this in the Settings routine
 * 5. Add controls on the Editor page to ensure that the Strings conform to the models
 * 		as in - date is in format MM/DD/YYYY (or acceptable alternative)
 * 6. Add Sorting and possibly filters to the list pane
 */
package com.garryburns.ExpenseIT;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;

@SuppressWarnings("serial")
public class ExpenseIT extends JFrame implements ActionListener {
 static ExpenseList expenseList;
 static CategoriesList categoriesList;
 private static int indexSelected = 1;
 private JPanel jplSuper;
 private static JList list;
 private static DefaultListModel titles;
 private static JButton jbAdd, jbEdit, jbDelete, jbUndo, jbSettings;
 private Expense deletedExpense;
 private int indexDeleted;
 private static JPanel jplMain;
 private static EditPane jplEditor;
 private static AddExpense addExpense;
 private final JLabel version = new JLabel("Version 0.2.3");
 
 private static JButton resetList;
 //TODO delete resetList and it's references - it's used only for testing purposes
 
 ExpenseIT() {
	// set up the Frame 
	super("ExpenseIT");
	setSize(320, 480);
	setLocation(400, 250);
	setResizable(false);
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
	//set up the expense list
	loadExpenseList();
	loadCategoriesList();

	jplEditor = new EditPane();
	addExpense = new AddExpense();
	
 	//set up the list UI
	titles = new DefaultListModel();
	//titles.addElement(new Object());//Add an element to override null pointer errors from starting in Firefox
	setTitles();
	list = new JList(titles);
	list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	list.setLayoutOrientation(JList.VERTICAL);
	list.setSelectedIndex(0);
	
	//set up the main Pane to hold the list and the buttons
	jplMain = new JPanel();
	JLabel jlbHeaders;
	String sortHeaders = "Date  ::  Category  ::  Location  ::  Amount";
	
	JScrollPane listScroller = new JScrollPane(list);
	listScroller.setPreferredSize(new Dimension(310, 300));
	
	jlbHeaders = new JLabel(sortHeaders);
	jbAdd = new JButton("Add");
	jbEdit = new JButton("Edit");
	jbDelete = new JButton("Delete");	
	enableDeleteButton();
	jbUndo = new JButton("Undo");
	jbUndo.setEnabled(false);					//Disable Undo until there is a deleted expense
	jbSettings = new JButton("Settings");
	jbSettings.setEnabled(false);				//Disable Settings until a settings set is created
	resetList = new JButton("Default List");
	
	jplMain.add(jlbHeaders);
	jplMain.add(listScroller);
	jplMain.add(jbAdd);
	jplMain.add(jbEdit);
	jplMain.add(jbDelete);
	jplMain.add(jbUndo);
	jplMain.add(jbSettings);
	jplMain.add(resetList);
	//End Main Pane Setup
	
	//Setup the Version Label Pane
	JPanel versionPane = new JPanel();
	versionPane.add(version);

	
	// add ActionListeners
	jbAdd.addActionListener(this);
	jbEdit.addActionListener(this);
	jbDelete.addActionListener(this);
	jbUndo.addActionListener(this);
	jbSettings.addActionListener(this);
	jplEditor.jbSave.addActionListener(this);
	jplEditor.jbCancel.addActionListener(this);
	addExpense.jbDone.addActionListener(this);
	addExpense.jbHome.addActionListener(this);
	resetList.addActionListener(this);
	
	jplSuper = new JPanel();
	jplSuper.setLayout(new BorderLayout());
	jplSuper.add(jplMain, BorderLayout.CENTER);
	jplSuper.add(versionPane, BorderLayout.PAGE_END);
	setContentPane(jplSuper);
	setVisible(true);	
 }  // End of Constructor
 

@Override
public void actionPerformed(ActionEvent evt) {
	 JButton jbSource = (JButton) evt.getSource();
	 
	 if(jbSource == resetList) {
		 expenseList = new ExpenseList();
		 enableDeleteButton();
		 jbUndo.setEnabled(false);
		 setTitles();
		 repaint();
	 }
	 if(jbSource == addExpense.jbHome) {
		 goHome();
	 }
	 if(jbSource == addExpense.jbDone) {
		 //Come home
		 indexSelected = expenseList.getSize() - 1;
		 goHome();
	 }
	 if(jbSource == jbAdd){
		 // cannot simply make a new instance of AddExpense because the action listeners
		 //	were set up for the old one and won't transfer to the new instance
		 setContentPane(addExpense);
		 setVisible(true);
		 repaint();
	 }
	 if(jbSource == jbEdit) {
		 //Move to the editor screen
		 if(list.getSelectedIndex() == -1) 		//Make sure an existing expense was selected
			 JOptionPane.showMessageDialog(getParent(), "Please choose an expense to edit.");
		 else{
			 indexSelected = list.getSelectedIndex();
			 populateEditor(indexSelected);
			 setContentPane(jplEditor);
			 setVisible(true);
			 repaint();
		 }
	 }
	 if(jbSource == jbDelete) {
		 deleteExpense();
		 enableDeleteButton();
	 }
	 if(jbSource == jbUndo){
		 undoDelete();
	 }
	 if(jbSource == jbSettings) {
		 //TODO create some settings
	 }
	 if(jbSource == jplEditor.jbCancel) {
		 //Return Home from the Editor with out saving changes
		 goHome();
	 }
	 if(jbSource == jplEditor.jbSave) {
		 //Save the changes made in the Editor and return Home
		 setExpense(indexSelected);
		 goHome();
	 }	
 }
 private void loadCategoriesList() {
	 File file = new File("ExpenseIT.cats");
	 // if there's a category list already saved in ExpenseIT.cat - load that list
	 if(file.exists()) {
		 try {
			 FileInputStream fi = new FileInputStream("ExpenseIT.cats");
			 ObjectInputStream oi = new ObjectInputStream(fi);
			 categoriesList = (CategoriesList) oi.readObject();
			 // TODO add some testing to ensure list is viable
			 oi.close();
		 } catch (Exception e) {
			 System.out.println("Error - " + e.toString());
		 }
	 } 
	 // if no previous list exists (there's no file) then setup a new default list
	 else { 
		 categoriesList = new CategoriesList();
	 }
 }
 private void loadExpenseList() {
	 File file = new File("ExpenseIT.exp");
	 // if there's a list already saved in ExpenseIT.exp - load that list
	 if(file.exists()) {
		 try {
			 FileInputStream fi = new FileInputStream("ExpenseIT.exp");
			 ObjectInputStream oi = new ObjectInputStream(fi);
			 expenseList = (ExpenseList) oi.readObject();
			 // TODO add some testing to ensure list is viable
			 oi.close();
		 } catch (Exception e) {
			 System.out.println("Error - " + e.toString());
		 }
	 } 
	 // if no previous list exists (there's no file) then setup a new default list
	 else { 
		 expenseList = new ExpenseList();
	 }
 }
 private void enableDeleteButton() {
	 if(expenseList.getSize() < 1)				//Disable Delete if there's nothing to delete
		jbDelete.setEnabled(false);
	 else {
		 jbDelete.setEnabled(true);
	 }
 }
 private void deleteExpense() {
	 //Delete the selected expense then show the new list
	 indexSelected = list.getSelectedIndex();
	 deletedExpense = (Expense) expenseList.remove(indexSelected);	//Remove Expense and store in case of undo
	 indexDeleted = indexSelected;				//Save the deleted expense's index
	 expenseList.saveList(expenseList);
	 jbUndo.setEnabled(true);
	 setTitles();
	 if(indexSelected < expenseList.getSize())
		 list.setSelectedIndex(indexSelected);
	 else list.setSelectedIndex(indexSelected - 1);
	 repaint();
 }
 private void undoDelete() {
	 jbUndo.setEnabled(false);
	 expenseList.add(indexDeleted, deletedExpense);
	 expenseList.saveList(expenseList);
	 setTitles();
	 list.setSelectedIndex(indexDeleted);
	 enableDeleteButton();
	 repaint();
 }
 private void setExpense(int index) {
	 Expense exp = new Expense(jplEditor.jtxDate.getText(), 
			 				   jplEditor.jtxCat.getText(), 
			 				   jplEditor.jtxLoc.getText(), 
			 				   jplEditor.jtxAmt.getText(), 
			 				   jplEditor.jtxNotes.getText());
	 expenseList.set(indexSelected, exp);
	 expenseList.saveList(expenseList);
 }
 private void populateEditor(int index) {
	 Expense editedExpense = (Expense) expenseList.get(index);
	 jplEditor.jtxCat.setText(editedExpense.getCategory());
	 jplEditor.jtxDate.setText(editedExpense.getDate());
	 jplEditor.jtxLoc.setText(editedExpense.getLocation());
	 jplEditor.jtxAmt.setText(editedExpense.getAmount());
	 jplEditor.jtxNotes.setText(editedExpense.getNotes());
 }
 private void setTitles() {
	 titles.clear();
	 for(int i = 0; i < expenseList.size(); ++i){
		 Expense exp = (Expense) expenseList.elementAt(i);
		 String title = (exp.getDate() + " :: " + exp.getCategory() + " :: " 
				 + exp.getLocation() + " :: " + exp.getAmount());
		 titles.addElement(title);
	 }
 }
 public void goHome() {
	 jbUndo.setEnabled(false);
	 setTitles();
	 list.setSelectedIndex(indexSelected);
	 enableDeleteButton();
	 setContentPane(jplSuper);
	 setVisible(true);
	 repaint();
 }
 public static void main(String[] args) {
	new ExpenseIT();
 }
}


@SuppressWarnings("serial")
class EditPane extends JPanel {
 JButton jbSave, jbCancel;
 JLabel jlCat, jlDate, jlLoc, jlAmt, jlNotes;
 JTextField jtxCat, jtxDate, jtxLoc, jtxAmt;
 JTextArea jtxNotes;
 
 EditPane() {
	 super();
	 setLayout(new BorderLayout());
	 
	 JPanel buttonHolder = new JPanel();
	 buttonHolder.setLayout(new BoxLayout(buttonHolder, BoxLayout.LINE_AXIS));
	 jbSave = new JButton("Save");
	 jbCancel = new JButton("Cancel");
	 buttonHolder.add(Box.createHorizontalGlue());
	 buttonHolder.add(jbCancel);
	 buttonHolder.add(jbSave);
	 
	 JPanel editHolder = new JPanel();
	 editHolder.setLayout(new BoxLayout(editHolder, BoxLayout.PAGE_AXIS));
	 jtxCat = new JTextField("Category", 20);
	 jtxCat.setAlignmentX(LEFT_ALIGNMENT);
	 jtxDate = new JTextField("Date", 20);
	 jtxDate.setAlignmentX(LEFT_ALIGNMENT);
	 jtxLoc = new JTextField("Location", 20);
	 jtxLoc.setAlignmentX(LEFT_ALIGNMENT);
	 jtxAmt = new JTextField("$Amount", 20);
	 jtxAmt.setAlignmentX(LEFT_ALIGNMENT);
	 jtxNotes = new JTextArea("Notes:", 13, 20);
	 jtxNotes.setLineWrap(true);
	 jtxNotes.setWrapStyleWord(true);
	 JScrollPane scrollPane = new JScrollPane(jtxNotes);
	 scrollPane.setAlignmentX(LEFT_ALIGNMENT);
	 
	 jlCat = new JLabel("Category:");
	 jlCat.setAlignmentX(LEFT_ALIGNMENT);
	 jlDate = new JLabel("Date:");
	 jlDate.setAlignmentX(LEFT_ALIGNMENT);
	 jlLoc = new JLabel("Location:");
	 jlLoc.setAlignmentX(LEFT_ALIGNMENT);
	 jlAmt = new JLabel("Amount:");
	 jlAmt.setAlignmentX(LEFT_ALIGNMENT);
	 jlNotes = new JLabel("Notes:");
	 jlNotes.setAlignmentX(LEFT_ALIGNMENT);
	 
	 
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlCat);
	 editHolder.add(jtxCat);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlDate);
	 editHolder.add(jtxDate);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlLoc);
	 editHolder.add(jtxLoc);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlAmt);
	 editHolder.add(jtxAmt);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlNotes);
	 editHolder.add(scrollPane);
	 
	 add(buttonHolder, BorderLayout.PAGE_START);
	 add(editHolder, BorderLayout.CENTER);
	 setVisible(true);
 }
}