/**
 * Date: 3/03/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Parent Classes: EditExpense.java, AddExpense.java
 * Referenced Classes: Expense.java
 * 
 * Class Version: 1.0
 * Change Log:
 * 03/08/2011	Consolidated Expense class into ExpenseList.java. 
 * 				Changed ExpenseList save file to 'ExpenseIT.exp'
 * 				Added method to the ExpenseList class to write itself to a file
 * 				Added method to the ExpenseList class to create a new expense (should keep expenses private) 
 * 03/03/2011	Class Created
 * 
 * Notes: 
 * ExpenseList class:
 * Used to store the expenses as they are created. 
 * It can write itself to a file so the created list will persist.
 * Expense class:
 * Each instance of this class represents an individual expense. It contains 
 * all the information required. For ease and portability all fields are stored
 * as strings.
 * Things TODO:
 * 1. Add controls to ensure that input strings conform to the data models
 * 		and are saved in the appropriate formats as in - 
 * 		date is in format MM/DD/YYYY (or acceptable alternative)
 * 		amounts are saved to the hundredth (Amount = DDD.cc)
 * 2. Review Code and clean up and streamline as necessary
 */
package com.garryburns.ExpenseIT;

import java.io.*;
import javax.swing.DefaultListModel;


class ExpenseList extends DefaultListModel implements Serializable {
 private static final long serialVersionUID = 1L;
 // Create a brand new expense list
 ExpenseList(){
	super();
	//Add demo expenses
	Expense demoExp1 = new Expense("03/11/2011", 
				"Office Supplies", 
				"Office Depot", 
				"85.00", 
				"Pense, pencils, etc.");
	Expense demoExp2 = new Expense("03/15/2011", 
				"Dining", 
				"Maki Zushi", 
				"158.00",
				"Cleveland Clients");
	Expense demoExp3 = new Expense("03/12/2011",
				"Office Supplies",
				"Costco",
				"230.00",
				"Printing supplies, bathroom & cleaning supplies");
	Expense demoExp4 = new Expense("02/15/2010",
			"Entertainment",
			"The Yardhouse",
			"455.00",
			"Birthday Celebrations");

	addElement(demoExp1);
	addElement(demoExp2);
	addElement(demoExp3);
	addElement(demoExp4);
	
	saveList(this);
 }
 
 void saveList(ExpenseList list) {
	 try {
		 FileOutputStream fo = new FileOutputStream("ExpenseIT.exp");
		 ObjectOutputStream oo = new ObjectOutputStream(fo);
		 oo.writeObject(list);
		 oo.close();
	 } catch (IOException e) {
		 System.out.println("Error - " + e.toString());
	 }
 }
 ExpenseList getList() {
	 ExpenseList list;
	 try {
		 FileInputStream fi = new FileInputStream("ExpenseIT.exp");
		 ObjectInputStream oi = new ObjectInputStream(fi);
		 list = (ExpenseList) oi.readObject();
		 // TODO add some testing to ensure list is viable
		 oi.close();
		 return list;
	 } catch (Exception e) {
		 System.out.println("Error - " + e.toString());
		 return null;
	 }
 }

 void addExpense(ExpenseList expList, String inputLocation, String inputDate, String inputCat,
		 		 String inputAmount, String inputNotes) {
	Expense exp = new Expense (inputDate, inputCat, inputLocation, inputAmount, inputNotes);
	expList.addElement(exp);
	expList.saveList(this);
 }
}

class Expense implements Serializable {
 private static final long serialVersionUID = 1L;
 private String date, category, location, amount, notes;
 
 // Constructor
 Expense() {};
 Expense(String inDate, String inCat, String inLoc, String inAmt, String inNotes) {
	 date = inDate;
	 category = inCat;
	 location = inLoc;
	 amount = inAmt;
	 notes = inNotes;
 }
 
 // set information
 void setDate(String inDate) {
	 date = inDate;
 }
 void setCategory(String inCat) {
	 category = inCat;
 }
 void setLocation(String inLoc) {
	 location = inLoc;
 }
 void setAmount(String inAmt) {
	 amount = inAmt;
 }
 void setNotes(String inNotes) {
	 notes = inNotes;
 }
 
 // read information
 String getDate() {
	 return date;
 }
 String getCategory() {
	 return category;
 }
 String getLocation() {
	 return location;
 }
 String getAmount() {
	 return amount;
 }
 String getNotes() {
	 return notes;
 }
}