/**
 * Date: 3/03/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Parent Classes: EditExpense.java, AddExpense.java
 * Referenced Classes: 
 * 
 * Class Version: 1.2
 * Change Log:
 * 03/09/2011	Modified Expense class to accept dates as Date objects (they are still stored as strings)
 * 03/08/2011	Consolidated Expense class into ExpenseList.java. 
 * 				Changed ExpenseList save file to 'ExpenseIT.exp'
 * 				Added method to the ExpenseList class to write itself to a file
 * 				Added method to the ExpenseList class to create a new expense (should keep expenses private) 
 * 03/03/2011	Class Created
 * 
 * Notes: 
 * ExpenseList class:
 * Used to store the expenses as they are created. 
 * It can write itself to a file so the created list will persist.
 * Expense class:
 * Each instance of this class represents an individual expense. It contains 
 * all the information required. For ease and portability all fields are stored
 * as strings.
 * Things TODO:
 * 1. Combine all items to be saved into a single file
 * 2. Review Code and clean up and streamline as necessary
 */
package com.garryburns.ExpenseIT;

import java.io.*;
import java.sql.Date;
import java.util.Vector;
import javax.swing.table.*;


class ExpenseTableModel extends AbstractTableModel implements Serializable {
 private static final long serialVersionUID = 1L;
 private String[] columnNames = {"Date", "Category", "Location", "Amount $"};
 private Vector<Expense> data;
 
 ExpenseTableModel() {
	data = new Vector<Expense>();
	//Add demo expenses
	Expense demoExp1 = new Expense(DateO.makeDate("03/11/2011"), 
				"Office Supplies", 
				"Office Depot", 
				"85.00", 
				"Pense, pencils, etc.");
	Expense demoExp2 = new Expense(DateO.makeDate("03/15/2011"), 
				"Dining", 
				"Maki Zushi", 
				"158.00",
				"Cleveland Clients");
	Expense demoExp3 = new Expense(DateO.makeDate("03/12/2011"),
				"Office Supplies",
				"Costco",
				"230.00",
				"Printing supplies, bathroom & cleaning supplies");
	Expense demoExp4 = new Expense(DateO.makeDate("02/15/2010"),
				"Entertainment",
				"The Yardhouse",
				"455.00",
				"Birthday Celebrations");

	data.add(demoExp1);
	data.add(demoExp2);
	data.add(demoExp3);
	data.add(demoExp4);
	
	ExpenseIT.save();
 }

 public Expense remove(int index) {
	Expense exp = data.remove(index);
	return exp;
 }
 @Override
 public int getColumnCount() {
	return columnNames.length;
 }

 @Override
 public int getRowCount() {
	return data==null ? 0 : data.size();
 }

 @Override
 public String getColumnName(int col) {
	return columnNames[col];
 }
 @Override
 public Object getValueAt(int row, int col) {
    if (row < 0 || row>=getRowCount())
        return "";
      Expense exp = (Expense) data.get(row);
      switch (col) {
        case 0: return exp.getDate();
        case 1: return exp.getCategory();
        case 2: return exp.getLocation();
        case 3: return Double.parseDouble(exp.getAmount());
      }
      return "";
 }
 @Override
 public Class<?> getColumnClass(int c) {
	return getValueAt(0, c).getClass();
 }
 @Override
 public boolean isCellEditable(int row, int col) {
	return false;
 }
 @Override
 public void setValueAt(Object value, int nRow, int nCol) {
    if (nRow < 0 || nRow>=getRowCount())
      return;
    Expense row = (Expense) data.get(nRow);

    switch (nCol) {
      case 0: 
    	  row.setDate(value.toString());
    	  ExpenseIT.save();
    	  break;
      case 1:
    	  row.setCategory(value.toString());
    	  break;
      case 2:
    	  row.setLocation(value.toString());
    	  break;
      case 3:
    	  row.setAmount(value.toString());
    	  break;
      default: break;
	    }
	  }

 static void saveList() {
	 ExpenseIT.save();
 }

 void addExpense(String inputLocation, Date inputDate, String inputCat,
		 		 String inputAmount, String inputNotes) {
	Expense exp = new Expense (inputDate, inputCat, inputLocation, inputAmount, inputNotes);
	data.addElement(exp);
	ExpenseIT.save();
 }
 void addExpense(Expense exp) {
	 data.addElement(exp);
	 ExpenseIT.save();
 }
 void addExpense(int index, Expense exp) {
	 data.insertElementAt(exp, index);
	 ExpenseIT.save();
 }

 Expense get(int indexSelected) {
	Expense exp = data.get(indexSelected);
	return exp;
}

 void set(int indexSelected, Expense exp) {
	data.set(indexSelected, exp);
	ExpenseIT.save();
 }
}

class Expense implements Serializable {
 private static final long serialVersionUID = 1L;
 private String date, category, location, amount, notes;
 
 // Constructor
 Expense() {};
 //create an expense with a Date date
 Expense(Date inDate, String inCat, String inLoc, String inAmt, String inNotes) {
	 date = DateO.makeString(inDate);
	 category = inCat;
	 location = inLoc;
	 amount = inAmt;
	 notes = inNotes;
 }
 //create an expense with a String date
 Expense(String inDate, String inCat, String inLoc, String inAmt, String inNotes) {
	 date = inDate;
	 category = inCat;
	 location = inLoc;
	 amount = inAmt;
	 notes = inNotes;
 }

 // set information
 void setDate(Date inDate) {
	 date = DateO.makeString(inDate);
 }
 void setDate(String inDate) {
	 date = inDate;
 }
 void setCategory(String inCat) {
	 category = inCat;
 }
 void setLocation(String inLoc) {
	 location = inLoc;
 }
 void setAmount(String inAmt) {
	 amount = inAmt;
 }
 void setNotes(String inNotes) {
	 notes = inNotes;
 }
 
 // read information
 String getDate() {
	 return date;
 }
 String getCategory() {
	 return category;
 }
 String getLocation() {
	 return location;
 }
 String getAmount() {
	 return amount;
 }
 String getNotes() {
	 return notes;
 }
}