import java.awt.event.*;
import java.awt.*;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import javax.swing.*;

public class AddExpense extends JPanel implements ActionListener {
 JButton jbHome; // Home button to go back the to Home screen of the overall program
 private AddSplashPane jplSplash;
 private AddEditPane jplEditor;
 private AddNotesPane jplNotes;
 private Expense newExpense;
 private CardLayout cards = new CardLayout();
 private final String SPLASH = "Splash";
 private final String EDITOR = "Editor";
 private final String NOTES = "Notes";
 
 public AddExpense() {
	 super();
	 setLayout(cards);
	 
	 newExpense = new Expense();
	 // set up cards
	 jplSplash = new AddSplashPane();
	 jplEditor = new AddEditPane();
	 jplNotes = new AddNotesPane();
	 
	 jbHome = jplSplash.jbBack;		// allows jplSplash to remain private to this class
	 
	 // add action listeners
	 jplSplash.jbNext.addActionListener(this);
	 for(int i = 0; i < jplEditor.jbCategories.length; ++i)
		 jplEditor.jbCategories[i].addActionListener(this);
	 jplNotes.jbSave.addActionListener(this);

	 // add the cards
	 add(jplSplash, SPLASH);
	 add(jplEditor, EDITOR);
	 add(jplNotes, NOTES);
	 
	 setVisible(true);
 }  // End of Constructor

 @Override
 public void actionPerformed(ActionEvent evt) {
	 JButton source = (JButton) evt.getSource();
	 String cardToShow = SPLASH;
	 
 	 if(source == jplSplash.jbNext) {
 		 // Save the location into newExpense
 		 newExpense.setLocation(getInputLocation());
 		 // ask for the date as you move from Splash to Editor
 		 String date = JOptionPane.showInputDialog(getParent(), "Date:");
 		 newExpense.setDate(date);
 	 	 //TODO find a better dialog option that can use dates and has no Cancel button
 		 cardToShow = EDITOR;
 	 }
	 if(source == jplNotes.jbSave) {		// save the data and return to Splash
		 newExpense.setNotes(getInputNotes()); // save the notes
		 saveExpense(newExpense);
		 cardToShow = SPLASH;
	 }
	 else {
		 for (int i = 0; i < jplEditor.jbCategories.length; ++i) {
			 if(source == jplEditor.jbCategories[i]) {
				 newExpense.setCategory(jplEditor.jbCategories[i].getText());
				 newExpense.setAmount(getInputAmount());
				 cardToShow = NOTES;
			 }
		 }
	 }
	 cards.show(this, cardToShow);
 }
 
 String getInputLocation() {
	 return jplSplash.getInLocation();
 }
 String getInputNotes() {
	 return jplNotes.getNotes();
 }
 String getInputAmount() {
	 return jplEditor.calc.getDisplayString();
 }
 
 void saveExpense(Expense exp) {
	 ExpenseIT.expenseList.addElement(exp);
	 ExpenseIT.expenseList.saveList(ExpenseIT.expenseList);
 }
}

class AddSplashPane extends JPanel {
 JLabel banner;
 JTextField jtxLocation;
 JButton jbNext, jbBack;
 Dimension textFieldSize = new Dimension(1000, 30);
 
 AddSplashPane() {
	super();
	//setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
	banner = new JLabel("Add a New Expense");

	jtxLocation = new JTextField("Where are you?", 25);
	jtxLocation.setMaximumSize(textFieldSize);
	
	JPanel buttonHolder = new JPanel();
	jbNext = new JButton("Next");
	jbBack = new JButton("Back");
	buttonHolder.add(jbNext);
	buttonHolder.add(jbBack);
	
	add(banner);
	add(Box.createVerticalStrut(100));
	add(jtxLocation);
	add(Box.createVerticalStrut(10));
	add(buttonHolder);
 }
 
 String getInLocation() {
	 return jtxLocation.getText();
 }
}

class AddEditPane extends JPanel {
 JLabel banner;
 JButton[] jbCategories = new JButton[4];
 JPanel jplButtonHolder;
 Calculator calc;
 
 AddEditPane() {
	super();
	setLayout(new BorderLayout());
	
	banner = new JLabel("ExpenseIT");
	calc = new Calculator();
	
	jplButtonHolder = new JPanel();
	jplButtonHolder.setLayout(new GridLayout(4, 1));
	
	jbCategories[0] = new JButton("Blue");
	jbCategories[1] = new JButton("Red");
	jbCategories[2] = new JButton("Green");
	jbCategories[3] = new JButton("Orange");

	jbCategories[0].setForeground(Color.blue);
	jbCategories[1].setForeground(Color.red);
	jbCategories[2].setForeground(Color.green);
	jbCategories[3].setForeground(Color.orange);
	
	for(int i = 0; i < jbCategories.length; ++i){
		jbCategories[i].setMaximumSize(new Dimension(20,20));
		jplButtonHolder.add(jbCategories[i]);
	}
		
	JPanel calcHolder = new JPanel();
	calcHolder.setLayout(new BorderLayout());
	calcHolder.add(calc, BorderLayout.PAGE_END);
	
	add(banner, BorderLayout.PAGE_START);
	add(calcHolder, BorderLayout.CENTER);
	add(jplButtonHolder, BorderLayout.LINE_END);	
 }
}

class AddNotesPane extends JPanel {
 JLabel banner;
 JButton jbSave;
 JScrollPane scrollPane;
 JTextArea jtxNotes;
 
 AddNotesPane() {
	super();
	setLayout(new BorderLayout());
	banner = new JLabel("ExpenseIT");
	jtxNotes = new JTextArea("Notes: ", 8, 20);
	scrollPane = new JScrollPane(jtxNotes);
	jbSave = new JButton("Save");
	
	JPanel topPanel = new JPanel();
	topPanel.setLayout(new BorderLayout());
	topPanel.add(banner, BorderLayout.LINE_START);
	topPanel.add(jbSave, BorderLayout.LINE_END);
	
	add(topPanel, BorderLayout.PAGE_START);
	add(scrollPane, BorderLayout.CENTER);
 }
 
 String getNotes() {
	 return jtxNotes.getText();
 }
}