/*
Name: Garry Burns
Website: http://www.garryburns.com

Description: Basic Java Calculator

Conventions Used in Source code
---------------------------------
	1. All JLabel components start with jlb*
	2. All JPanel components start with jpl*
	3. All JMenu components start with jmenu*
	4. All JMenuItem components start with jmenuItem*
	5. All JDialog components start with jdlg*
	6. All JButton components start with jbn*
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class Calculator extends JPanel implements ActionListener{
	// Variables
	final int MAX_INPUT_LENGTH = 20;
	final int INPUT_MODE = 0;
	final int RESULT_MODE = 1;
	final int ERROR_MODE = 2;
	int displayMode;
	final String ADD = "+";
	final String SUBTRACT = "-";
	final String MULTIPLY = "*";
	final String DIVIDE = "/";
	final String EQUALS = "0";

	boolean clearOnNextDigit, percent;
	double lastNumber;
	String lastOperator;

	private JLabel jlbOutput;
	private JButton jbnButtons[];
	private JPanel jplMaster, jplBackSpace, jplControl;
	
	/*
	 * Font(String name, int style, int size)
      Creates a new Font from the specified name, style and point size.
	 */
	
	Font f12 = new Font("Times New Roman", Font.PLAIN, 12);
	Font f121 = new Font("Times New Roman", Font.BOLD, 12);
	
	// Constructor 
	public Calculator() 
	{
		super();
		jplMaster = new JPanel();
		
		//Set frame layout manager
		setLayout(new BorderLayout());
		setBackground(Color.gray);

		jlbOutput = new JLabel("0");
		jlbOutput.setPreferredSize(new Dimension(100, 30));
		jlbOutput.setHorizontalTextPosition(JLabel.RIGHT);
		jlbOutput.setBackground(Color.WHITE);
		jlbOutput.setOpaque(true);
		
		//GridLayout(int rows, int cols, int hgap, int vgap) 
		// container for Buttons
		JPanel jplButtons = new JPanel();

		// Create numeric Buttons 0-9
		jbnButtons = new JButton[19];
		for (int i=0; i<=9; i++)
			jbnButtons[i] = new JButton("" + i);

		// Create operator Buttons
		jbnButtons[10] = new JButton(".");
		jbnButtons[11] = new JButton("=");
		jbnButtons[12] = new JButton(DIVIDE);
		jbnButtons[13] = new JButton(MULTIPLY);
		jbnButtons[14] = new JButton(SUBTRACT);
		jbnButtons[15] = new JButton(ADD);

		
		jplBackSpace = new JPanel();
		jplBackSpace.setLayout(new GridLayout(1, 1, 2, 2));

		jbnButtons[16] = new JButton("Backspace");
		jplBackSpace.add(jbnButtons[16]);

		jplControl = new JPanel();
		jplControl.setLayout(new GridLayout(1, 2, 2 ,2));

		jbnButtons[17] = new JButton(" CE ");
		jbnButtons[18] = new JButton("C");

		jplControl.add(jbnButtons[17]);
		jplControl.add(jbnButtons[18]);

		//Setting all Numbered Buttons to Blue. Operators to Red
		for (int i=0; i<jbnButtons.length; i++)	{
			jbnButtons[i].setFont(f12);
			if(i < 16)
			jbnButtons[i].setPreferredSize(new Dimension(50,50));

			if (i<10)
				jbnButtons[i].setForeground(Color.blue);
				
			else
				jbnButtons[i].setForeground(Color.red);
		}
	
		// Set panel layout manager for a 4 by 5 grid
		jplButtons.setLayout(new GridLayout(4, 4, 2, 2));
		
		//Add buttons to keypad panel starting at top left
		// First row
		for(int i=7; i<=9; i++)				// add 7-9
			jplButtons.add(jbnButtons[i]);
		
		jplButtons.add(jbnButtons[12]);		// add  /
		
		// Second row
		for(int i=4; i<=6; i++)				// add 4-6
			jplButtons.add(jbnButtons[i]);
		
		jplButtons.add(jbnButtons[13]);		// add *

		// Third row
		for( int i=1; i<=3; i++)			// add 1-3
			jplButtons.add(jbnButtons[i]);
		
		jplButtons.add(jbnButtons[14]);		// add -
		
		//Fourth Row
		jplButtons.add(jbnButtons[0]);		// add 0
		jplButtons.add(jbnButtons[10]);		// add .
		jplButtons.add(jbnButtons[11]);		// add =
		jplButtons.add(jbnButtons[15]);		// add +
		
		jplMaster.setLayout(new BorderLayout());
		jplMaster.add(jplBackSpace, BorderLayout.WEST);
		jplMaster.add(jplControl, BorderLayout.EAST);
		jplMaster.add(jplButtons, BorderLayout.SOUTH);

		// Add components to Super
		add(jlbOutput, BorderLayout.NORTH);
		add(jplMaster, BorderLayout.SOUTH);
		requestFocus();
		
		//activate ActionListeners
		for (int i=0; i<jbnButtons.length; i++){
			jbnButtons[i].addActionListener(this);
		}
		
		clearAll();

	}	
	//End of Constructor

	// Here's the actions to perform when buttons are pushed
	public void actionPerformed(ActionEvent e){
		double result = 0;

		// Search for the button pressed until end of array or key found
		for (int i=0; i<jbnButtons.length; i++)
		{
			if(e.getSource() == jbnButtons[i])
			{
				switch(i)
				{
					case 0:
						addDigitToDisplay(i);
						break;

					case 1:
						addDigitToDisplay(i);
						break;

					case 2:
						addDigitToDisplay(i);
						break;

					case 3:
						addDigitToDisplay(i);
						break;

					case 4:
						addDigitToDisplay(i);
						break;

					case 5:
						addDigitToDisplay(i);
						break;

					case 6:
						addDigitToDisplay(i);
						break;

					case 7:
						addDigitToDisplay(i);
						break;

					case 8:
						addDigitToDisplay(i);
						break;

					case 9:
						addDigitToDisplay(i);
						break;

					case 10:	// decimal point
						addDecimalPoint();
						break;

					case 11:	// =
						processEquals();
						break;

					case 12:	// divide
						processOperator(DIVIDE);
						break;

					case 13:	// *
						processOperator(MULTIPLY);
						break;

					case 14:	// -
						processOperator(SUBTRACT);
						break;

					case 15:	// +
						processOperator(ADD);
						break;

					case 16:	// backspace
						if (displayMode != ERROR_MODE){
							setDisplayString(getDisplayString().substring(0,
										getDisplayString().length() - 1));
							
							if (getDisplayString().length() < 1)
								setDisplayString("0");
						}
						break;

					case 17:	// CE
						clearExisting();
						break;

					case 18:	// C
						clearAll();
						break;
				}
			}
		}
	}

	
	void setDisplayString(String s){
		jlbOutput.setText(s);
	}

	String getDisplayString (){
		return jlbOutput.getText();
	}

	void addDigitToDisplay(int digit){
		if (clearOnNextDigit)
			setDisplayString("");

		String inputString = getDisplayString();
		
		if (inputString.indexOf("0") == 0){  			// true if the number starts with 0
			inputString = inputString.substring(1);		// drop the leading 0 
		}

		if ((!inputString.equals("0") || digit > 0)  && inputString.length() < MAX_INPUT_LENGTH){
			setDisplayString(inputString + digit);
		}
		

		displayMode = INPUT_MODE;
		clearOnNextDigit = false;
	}

	void addDecimalPoint(){
		displayMode = INPUT_MODE;

		if (clearOnNextDigit)
			setDisplayString("");

		String inputString = getDisplayString();
	
		// If the input string already contains a decimal point, don't
		//  do anything to it.
		if (inputString.indexOf(".") < 0)
			setDisplayString(inputString + ".");
	}

	void clearAll()	{
		setDisplayString("0");
		lastOperator = EQUALS;
		lastNumber = 0;
		displayMode = INPUT_MODE;
		clearOnNextDigit = true;
	}

	void clearExisting(){
		setDisplayString("0");
		clearOnNextDigit = true;
		displayMode = INPUT_MODE;
	}

	double getNumberInDisplay()	{
		String input = jlbOutput.getText();
		return Double.parseDouble(input);
	}

	void processOperator(String op) {
		if (displayMode != ERROR_MODE)
		{
			double numberInDisplay = getNumberInDisplay();

			if (!lastOperator.equals(EQUALS))	
			{
				try
				{
					double result = processLastOperator();
					displayResult(result);
					lastNumber = result;
				}

				catch (DivideByZeroException e)
				{
				}
			}
		
			else
			{
				lastNumber = numberInDisplay;
			}
			
			clearOnNextDigit = true;
			lastOperator = op;
		}
	}

	void processEquals(){
		double result = 0;

		if (displayMode != ERROR_MODE){
			try			
			{
				result = processLastOperator();
				displayResult(result);
			}
			
			catch (DivideByZeroException e)	{
				displayError("Cannot divide by zero!");
			}

			lastOperator = EQUALS;
		}
	}

	double processLastOperator() throws DivideByZeroException {
		double result = 0;
		double numberInDisplay = getNumberInDisplay();

		if (lastOperator.equals(DIVIDE))
		{
			if (numberInDisplay == 0)
				throw (new DivideByZeroException());

			result = lastNumber / numberInDisplay;
		}
			
		if (lastOperator.equals(MULTIPLY))
			result = lastNumber * numberInDisplay;

		if (lastOperator.equals(SUBTRACT))
			result = lastNumber - numberInDisplay;

		if (lastOperator.equals(ADD))
			result = lastNumber + numberInDisplay;

		return result;
	}

	void displayResult(double result){
		setDisplayString(Double.toString(result));
		lastNumber = result;
		displayMode = RESULT_MODE;
		clearOnNextDigit = true;
	}

	void displayError(String errorMessage){
		setDisplayString(errorMessage);
		lastNumber = 0;
		displayMode = ERROR_MODE;
		clearOnNextDigit = true;
	}	
}		//End of Swing Calculator Class.

class DivideByZeroException extends Exception{
	public DivideByZeroException()
	{
		super();
	}
	
	public DivideByZeroException(String s)
	{
		super(s);
	}
}