/**
 * Date: 3/03/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Version: 0.2
 * Parent File Names: EditExpense.java
 * Referenced Files: 
 * 			Calculator.java
 * 			Expense.java
 * 			ExpenseList.java
 * 
 * Class Version: 0.2
 * Change Log:
 * 03/03/2011	Class Created
 * 03/08/2011	Modified class to function with ExpenseIT version 0.2
 * 
 * Notes: 
 * This is the class used to create new expenses and add them to the expense list.
 * It incorporates all UI and code functions. Only home and save buttons should be
 * called from outside this class.
 * It is set up as a wizard to walk the user through expense creation.
 * 
 * Things TODO:
 * 1. Clean up UI
 * 2. Add controls to ensure that input strings conform to the data models
 * 		and are saved in the appropriate formats as in - 
 * 		date is in format MM/DD/YYYY (or acceptable alternative)
 * 		amounts are saved to the hundredth (Amount = DDD.cc)
 * 3. Create a custom dialog for entering the date
 * 4. Create a drop down menu for the Location input field that includes previously used
 * 		locations and accepts new values
 * 5. Create an 'Other' categories button which allows a user to:
 * 		choose buttons not listed on the UI
 * 		create new categories and use them
 * 6. Possibly consolidate the add Notes portion to the Calculator pane
 * 7. Review Code and clean up and streamline as necessary
 */


import java.awt.event.*;
import java.awt.*;

import javax.swing.*;

@SuppressWarnings("serial")
public class AddExpense extends JPanel implements ActionListener {
 public JButton jbHome, jbDone;		// generic buttons that allow jbBack and jbSave to remain private
 public Expense newExpense;
 private AddSplashPane jplSplash;
 private CalculatorPane jplCalculator;
 private NotesPane jplNotes;
 private CardLayout cards = new CardLayout();
 private final String SPLASH = "Splash";
 private final String CALCULATOR = "Calculator";
 private final String NOTES = "Notes";
 
 public AddExpense() {
	 super();
	 setLayout(cards);

	 newExpense = new Expense();
	 // set up cards
	 jplSplash = new AddSplashPane();
	 jplCalculator = new CalculatorPane();
	 jplNotes = new NotesPane();
	 
	 jbHome = jplSplash.jbBack;		// allows jplSplash to remain private to this class
	 jbDone = jplNotes.jbSave;
	 
	 // add action listeners
	 jplSplash.jbNext.addActionListener(this);
	 for(int i = 0; i < jplCalculator.jbCategories.length; ++i)
		 jplCalculator.jbCategories[i].addActionListener(this);
	 jplNotes.jbSave.addActionListener(this);

	 // add the cards
	 add(jplSplash, SPLASH);
	 add(jplCalculator, CALCULATOR);
	 add(jplNotes, NOTES);
	 
	 setVisible(true);
 }  // End of Constructor

 @Override
 public void actionPerformed(ActionEvent evt) {
	 JButton source = (JButton) evt.getSource();
	 String cardToShow = SPLASH;
	 
 	 if(source == jplSplash.jbNext) {
 		 // Save the location into newExpense
 		 newExpense.setLocation(getInputLocation());
 		 // ask for the date as you move from Splash to Editor
 		 String date = JOptionPane.showInputDialog(getParent(), "Date:");
 		 newExpense.setDate(date);
 	 	 //TODO find a better dialog option that can use dates and has no Cancel button
 		 cardToShow = CALCULATOR;
 	 }
	 if(source == jplNotes.jbSave) {		
		 // newExpense is saved on the EditExpense side
		 cardToShow = SPLASH;
	 }
	 else {
		 for (int i = 0; i < jplCalculator.jbCategories.length; ++i) {
			 if(source == jplCalculator.jbCategories[i]) {
				 newExpense.setCategory(jplCalculator.jbCategories[i].getText());
				 newExpense.setAmount(getInputAmount());
				 cardToShow = NOTES;
			 }
		 }
	 }
	 cards.show(this, cardToShow);
 }
 
 String getInputLocation() {
	 return jplSplash.getInLocation();
 }
 String getInputNotes() {
	 return jplNotes.getNotes();
 }
 String getInputAmount() {
	 return jplCalculator.calc.getDisplayString();
 }
 void clearFields() {
	 jplSplash.jtxLocation.setText("Where are you?");
	 jplCalculator.calc.setDisplayString("0");
	 jplNotes.jtxNotes.setText("Notes: ");
 }
}

@SuppressWarnings("serial")
class AddSplashPane extends JPanel {
 JLabel banner, jlbLocation;
 JTextField jtxLocation;
 JButton jbNext, jbBack;
 Dimension textFieldSize = new Dimension(1000, 30);
 
 AddSplashPane() {
	super();
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
	banner = new JLabel("Add a New Expense");
	banner.setAlignmentX(LEFT_ALIGNMENT);
	jlbLocation = new JLabel("Location:");
	jlbLocation.setAlignmentX(LEFT_ALIGNMENT);
	
	jtxLocation = new JTextField("Where are you?", 25);
	jtxLocation.setMaximumSize(textFieldSize);
	jtxLocation.setAlignmentX(LEFT_ALIGNMENT);
	
	JPanel buttonHolder = new JPanel();
	jbNext = new JButton("Next");
	jbBack = new JButton("Back");
	buttonHolder.add(jbNext);
	buttonHolder.add(jbBack);
	buttonHolder.setAlignmentX(LEFT_ALIGNMENT);
	
	add(banner);
	add(Box.createVerticalStrut(100));
	add(jlbLocation);
	add(jtxLocation);
	add(Box.createVerticalStrut(10));
	add(buttonHolder);
 }
 
 String getInLocation() {
	 return jtxLocation.getText();
 }
}

@SuppressWarnings("serial")
class CalculatorPane extends JPanel {
 JLabel banner;
 JButton[] jbCategories = new JButton[4];
 JPanel jplButtonHolder;
 Calculator calc;
 
 CalculatorPane() {
	super();
	setLayout(new BorderLayout());
	
	banner = new JLabel("ExpenseIT");
	calc = new Calculator();
	
	jplButtonHolder = new JPanel();
	jplButtonHolder.setLayout(new GridLayout(4, 1));
	
	jbCategories[0] = new JButton("Blue");
	jbCategories[1] = new JButton("Red");
	jbCategories[2] = new JButton("Green");
	jbCategories[3] = new JButton("Orange");

	jbCategories[0].setForeground(Color.blue);
	jbCategories[1].setForeground(Color.red);
	jbCategories[2].setForeground(Color.green);
	jbCategories[3].setForeground(Color.orange);
	
	for(int i = 0; i < jbCategories.length; ++i){
		jbCategories[i].setMaximumSize(new Dimension(20,20));
		jplButtonHolder.add(jbCategories[i]);
	}
		
	JPanel calcHolder = new JPanel();
	calcHolder.setLayout(new BorderLayout());
	calcHolder.add(calc, BorderLayout.PAGE_END);
	
	add(banner, BorderLayout.PAGE_START);
	add(calcHolder, BorderLayout.CENTER);
	add(jplButtonHolder, BorderLayout.LINE_END);	
 }
}

@SuppressWarnings("serial")
class NotesPane extends JPanel {
 JLabel banner;
 JButton jbSave;
 JScrollPane scrollPane;
 JTextArea jtxNotes;
 
 NotesPane() {
	super();
	setLayout(new BorderLayout());
	banner = new JLabel("ExpenseIT");
	jtxNotes = new JTextArea("Notes: ", 8, 20);
	scrollPane = new JScrollPane(jtxNotes);
	jbSave = new JButton("Save");
	
	JPanel topPanel = new JPanel();
	topPanel.setLayout(new BorderLayout());
	topPanel.add(banner, BorderLayout.LINE_START);
	topPanel.add(jbSave, BorderLayout.LINE_END);
	
	add(topPanel, BorderLayout.PAGE_START);
	add(scrollPane, BorderLayout.CENTER);
 }
 
 String getNotes() {
	 return jtxNotes.getText();
 }
}