/**
 * Date: 3/03/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Version: 0.2
 * Parent File Names: N/A
 * Subfile Names: 
 * 			AddExpense.java
 * 			Calculator.java
 * 			Expense.java
 * 			ExpenseList.java
 * 
 * Class Version: 0.2
 * Change Log:
 * 03/03/2011	Class Created
 * 03/08/2011	Consolidated EditPane and Home Frame into this class
 * 
 * Notes: 
 * This is the main  class for ExpenseIT Version 0.2
 * it makes the expense list pane the home page of the whole program.
 * 
 * Things TODO:
 * 1. Clean up UI
 * 2. Add settings control and functions
 * 3. Add some testing controls to the loadList() method to ensure the 
 * 		file loaded is acceptable
 * 4. Add an Undo button that restores and expense if it's been accidently deleted
 * 5. Add an about page
 * 6. Add controls on the Editor page to ensure that the Strings conform to the models
 * 		as in - date is in format MM/DD/YYYY (or acceptable alternative)
 * 7. Add Sorting and possibly filters to the list pane
 */



import java.awt.event.*;
import java.awt.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;

import javax.swing.*;

@SuppressWarnings("serial")
public class ExpenseIT extends JFrame implements ActionListener {
 public static ExpenseList expenseList;
 private static int indexSelected = 1;
 private JPanel jplSuper;
 private static JList list;
 private static DefaultListModel titles;
 private static JButton jbAdd, jbEdit, jbDelete, jbSettings;
 private static JPanel jplMain;
 private static EditEditPane jplEditor;
 private static AddExpense addExpense;
 private final JLabel version = new JLabel("Version 0.2");
 
 private static JButton resetList;
 //TODO delete resetList and it's references - it's used only for testing purposes
 
 ExpenseIT() {
	// set up the Frame 
	super("ExpenseIT");
	setSize(320, 480);
	setLocation(400, 250);
	setResizable(false);
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
	//set up the expense list
	loadList();

	jplEditor = new EditEditPane();
	addExpense = new AddExpense();
	
 	//set up the list UI
	titles = new DefaultListModel();
	titles.addElement(new Object());		// add an element to override null pointer errors
	setTitles();
	list = new JList(titles);
	list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	list.setLayoutOrientation(JList.VERTICAL);
	list.setSelectedIndex(0);
	
	//set up the main Pane to hold the list and the buttons
	jplMain = new JPanel();
	JLabel jlbHeaders;
	String sortHeaders = "Date  ::  Category  ::  Location  ::  Amount";
	
	JScrollPane listScroller = new JScrollPane(list);
	listScroller.setPreferredSize(new Dimension(310, 300));
	
	jlbHeaders = new JLabel(sortHeaders);
	jbAdd = new JButton("Add");
	jbEdit = new JButton("Edit");
	jbDelete = new JButton("Delete");	
	enableDeleteButton();
	jbSettings = new JButton("Settings");
	jbSettings.setEnabled(false);				//Disable Settings until a settings set is created
	resetList = new JButton("Default List");
	
	jplMain.add(jlbHeaders);
	jplMain.add(listScroller);
	jplMain.add(jbAdd);
	jplMain.add(jbEdit);
	jplMain.add(jbDelete);
	jplMain.add(jbSettings);
	jplMain.add(resetList);
	//End Main Pane Setup
	
	//Setup the Version Label Pane
	JPanel versionPane = new JPanel();
	versionPane.add(version);

	
	// add ActionListeners
	jbAdd.addActionListener(this);
	jbEdit.addActionListener(this);
	jbDelete.addActionListener(this);
	jbSettings.addActionListener(this);
	jplEditor.jbSave.addActionListener(this);
	jplEditor.jbCancel.addActionListener(this);
	addExpense.jbDone.addActionListener(this);
	addExpense.jbHome.addActionListener(this);
	resetList.addActionListener(this);
	
	jplSuper = new JPanel();
	jplSuper.setLayout(new BorderLayout());
	jplSuper.add(jplMain, BorderLayout.CENTER);
	jplSuper.add(versionPane, BorderLayout.PAGE_END);
	setContentPane(jplSuper);
	setVisible(true);	
 }  // End of Constructor
 
 @Override
 public void actionPerformed(ActionEvent evt) {
	 JButton jbSource = (JButton) evt.getSource();
	 
	 if(jbSource == resetList) {
		 expenseList = new ExpenseList();
		 setTitles();
		 repaint();
	 }
	 if(jbSource == addExpense.jbHome) {
		 goHome();
	 }
	 if(jbSource == addExpense.jbDone) {
		 //Save the newExpense to the list
		 addExpense.newExpense.setNotes(addExpense.getInputNotes());
		 expenseList.addElement(addExpense.newExpense);
		 expenseList.saveList(expenseList);
		 addExpense.clearFields();					//reset the fields for a new expense
		 //Come home
		 goHome();
	 }
	 if(jbSource == jbAdd){
		 //create a new instance of newExpense 
		 addExpense.newExpense = new Expense();
		 // cannot simply make a new instance of AddExpense because the action listeners
		 //	were set up for the old one and won't transfer to the new instance
		 setContentPane(addExpense);
		 setVisible(true);
		 repaint();
	 }
	 if(jbSource == jbEdit) {
		 //Move to the editor screen
		 if(list.getSelectedIndex() == -1) 		//Make sure an existing expense was selected
			 JOptionPane.showMessageDialog(getParent(), "Please choose an expense to edit.");
		 else{
			 indexSelected = list.getSelectedIndex();
			 populateEditor(indexSelected);
			 setContentPane(jplEditor);
			 setVisible(true);
			 repaint();
		 }
	 }
	 if(jbSource == jbDelete) {
		 deleteExpense();
		 enableDeleteButton();
	 }
	 if(jbSource == jbSettings) {
		 //TODO create some settings
	 }
	 if(jbSource == jplEditor.jbCancel) {
		 //Return Home from the Editor with out saving changes
		 goHome();
	 }
	 if(jbSource == jplEditor.jbSave) {
		 //Save the changes made in the Editor and return Home
		 setExpense(indexSelected);
		 goHome();
	 }	
 }
 
 private void loadList() {
	 File file = new File("ExpenseIT.dat");
	 // if there's a list already saved in ExpenseIT.dat - load that list
	 if(file.exists()) {
		 try {
			 FileInputStream fi = new FileInputStream("ExpenseIT.dat");
			 ObjectInputStream oi = new ObjectInputStream(fi);
			 expenseList = (ExpenseList) oi.readObject();
			 // TODO add some testing to ensure list is viable
			 oi.close();
		 } catch (Exception e) {
			 System.out.println("Error - " + e.toString());
		 }
	 } 
	 // if no previous list exists (there's no file) then setup a new default list
	 else { 
		 expenseList = new ExpenseList();
	 }
 }
 private void enableDeleteButton() {
	 if(expenseList.getSize() < 1)				//Disable Delete if there's nothing to delete
		jbDelete.setEnabled(false);
	 else {
		 jbDelete.setEnabled(true);
	 }
 }
 private void deleteExpense() {
	 //Delete the selected expense then show the new list
	 indexSelected = list.getSelectedIndex();
	 expenseList.remove(indexSelected);	 
	 expenseList.saveList(expenseList);
	 setTitles();
	 list.setSelectedIndex(0);
	 repaint();
 }
 private void setExpense(int index) {
	 Expense exp = new Expense(jplEditor.jtxDate.getText(), 
			 				   jplEditor.jtxCat.getText(), 
			 				   jplEditor.jtxLoc.getText(), 
			 				   jplEditor.jtxAmt.getText(), 
			 				   jplEditor.jtxNotes.getText());
	 expenseList.set(indexSelected, exp);
	 expenseList.saveList(expenseList);
 }
 private void populateEditor(int index) {
	 Expense editedExpense = (Expense) expenseList.get(index);
	 jplEditor.jtxCat.setText(editedExpense.getCategory());
	 jplEditor.jtxDate.setText(editedExpense.getDate());
	 jplEditor.jtxLoc.setText(editedExpense.getLocation());
	 jplEditor.jtxAmt.setText(editedExpense.getAmount());
	 jplEditor.jtxNotes.setText(editedExpense.getNotes());
 }
 private void setTitles() {
	 titles.clear();
	 for(int i = 0; i < expenseList.size(); ++i){
		 Expense exp = (Expense) expenseList.elementAt(i);
		 String title = (exp.getDate() + " :: " + exp.getCategory() + " :: " 
				 + exp.getLocation() + " :: " + exp.getAmount());
		 titles.addElement(title);
	 }
 }

 private void goHome() {
	 setTitles();
	 list.setSelectedIndex(indexSelected);
	 enableDeleteButton();
	 setContentPane(jplSuper);
	 setVisible(true);
	 repaint();
 }
 
 public static void main(String[] args) {
		@SuppressWarnings("unused")
		ExpenseIT tester = new ExpenseIT();
	 }
}



@SuppressWarnings("serial")
class EditEditPane extends JPanel {
 JButton jbSave, jbCancel;
 /**
 * 
 */
JTextField jtxCat, jtxDate, jtxLoc, jtxAmt;
 JTextArea jtxNotes;
 Dimension textFieldSize = new Dimension(1000, 30);
 
 EditEditPane() {
	 super();
	 setLayout(new BorderLayout());
	 
	 JPanel buttonHolder = new JPanel();
	 buttonHolder.setLayout(new BoxLayout(buttonHolder, BoxLayout.LINE_AXIS));
	 jbSave = new JButton("Save");
	 jbCancel = new JButton("Cancel");
	 buttonHolder.add(Box.createHorizontalGlue());
	 buttonHolder.add(jbCancel);
	 buttonHolder.add(jbSave);
	 
	 JPanel editHolder = new JPanel();
	 editHolder.setLayout(new BoxLayout(editHolder, BoxLayout.PAGE_AXIS));
	 jtxCat = new JTextField("Category");
	 jtxCat.setMaximumSize(textFieldSize);
	 jtxDate = new JTextField("Date");
	 jtxDate.setMaximumSize(textFieldSize);
	 jtxLoc = new JTextField("Location");
	 jtxLoc.setMaximumSize(textFieldSize);
	 jtxAmt = new JTextField("$Amount");
	 jtxAmt.setMaximumSize(textFieldSize);
	 jtxNotes = new JTextArea("Notes:", 2, 8);
	 JScrollPane scrollPane = new JScrollPane(jtxNotes);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jtxCat);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jtxDate);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jtxLoc);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jtxAmt);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(scrollPane);
	 
	 add(buttonHolder, BorderLayout.PAGE_START);
	 add(editHolder, BorderLayout.CENTER);
	 setVisible(true);
 }
}