/**
 * Date: 3/03/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Parent File Names: EditExpense.java
 * Referenced Files: 
 * 			Calculator.java
 * 			CategoryList.java
 * 			ExpenseList.java
 * 
 * Class Version: 0.5
 * Change Log:
 * 03/11/2011	Modified dialog panes - they no longer do weird things when use cancels or x's out
 * 03/09/2011	Implemented CategoriesList class - CategoriesList.java
 * 				Added data control methods to verify proper expense inputs 
 * 03/08/2011	Modified Categories buttons; Consolidated Notes and Calculator Panes
 * 03/07/2011	Modified class to function with ExpenseIT version 0.2
 * 03/03/2011	Class Created
 * 
 * Notes: 
 * This is the class used to create new expenses and add them to the expense list.
 * It incorporates all UI and code functions. Only home and save buttons should be
 * called from outside this class.
 * It is set up as a wizard to walk the user through expense creation.
 * 
 * Things TODO:
 * 1. Clean up UI
 * 3. Create a drop down menu for the Location input field that includes previously used
 * 		locations and accepts new values
 * 4. Create an 'Other' categories button which allows a user to:
 * 		create new categories and use them
 * 5. Review Code and clean up and streamline as necessary
 */
package com.garryburns.ExpenseIT;

import java.awt.*;
import java.awt.event.*;
import java.sql.*;

import javax.swing.*;

@SuppressWarnings("serial")
class AddExpense extends JPanel implements ActionListener {
 JButton jbHome, jbDone;		// generic buttons that allow jbBack and jbSave to remain private
 private JButton[] jbDefaultCats = new JButton[5];
 private static String[] allCategories;
 private String category;
 private static Date date;
 private AddSplashPane jplSplash;
 private CalculatorPane jplEditor;
 private CardLayout cards = new CardLayout();
 private final String SPLASH = "Splash";
 private final String CALCULATOR = "Calculator";
 
 AddExpense() {
	 super();
	 setLayout(cards);
	 
	 // set up cards
	 jplSplash = new AddSplashPane();
	 jplEditor = new CalculatorPane();
	 
	 jbHome = jplSplash.jbBack;		// allows jplSplash to remain private to this class
	 jbDone = new JButton("Done");
	 
	 loadCatNames();
	 
	 //Create local buttons based on the Editor Pane's category buttons
	 for(int i = 0; i < jbDefaultCats.length; ++i) {
		 jbDefaultCats[i] = jplEditor.jbCategories[i];
	 }
		 
	 // add action listeners
	 jplSplash.jbNext.addActionListener(this);
	 for(int i = 0; i < jbDefaultCats.length; ++i)
		 jbDefaultCats[i].addActionListener(this);

	 // add the cards
	 add(jplSplash, SPLASH);
	 add(jplEditor, CALCULATOR);
	 
	 setVisible(true);
 }  // End of Constructor

 @Override
 public void actionPerformed(ActionEvent evt) {
	 JButton source = (JButton) evt.getSource();
	 
 	 if(source == jplSplash.jbNext) {
 		 //Ask the Date and move to CalculatorPane
 		 askForDate();
  	 }
	 else {
		 for (int i = 0; i < jbDefaultCats.length; ++i) {
			 //Loop through the Category buttons
			 if(source == jbDefaultCats[i]) {
				 if(source == jbDefaultCats[jbDefaultCats.length - 1]){
					//Other Button
					askForOther();
					return;
				 }
				 else {
					 category = jbDefaultCats[i].getActionCommand();
					 saveAndGoHome();
					 cards.show(this, SPLASH);
					 return;
				 }

			 }
		 }
	 }
 }

 private void askForDate() {
	 String input = JOptionPane.showInputDialog(getParent(), 
			 						"Date: MM/DD/YYYY or MMDDYY", DateO.today());
	 if ((input != null) && (input.length() > 0)) {
		 date = DateO.makeDate(input);
		 cards.show(this, CALCULATOR);
		 return;
	 }
	 else {
		 jbHome.doClick(); 
		 clearFields();
		 cards.show(this, SPLASH);
		 return;
	 }
}

private void askForOther() {
	  String input = (String) JOptionPane.showInputDialog(getParent(), 
			 				"Choose a Category:",
			 				"Choose a Category",
			 				JOptionPane.QUESTION_MESSAGE,
			 				null,
			 				allCategories, 
			 				allCategories[0]);
	  if ((input != null) && (input.length() > 0)) {
		  category = input;
		  saveAndGoHome();
		  cards.show(this, SPLASH);
		  return;
      } 
	  else { 
			cards.show(this, CALCULATOR);
			repaint();
	  }
}

static void loadCatNames() {
	 allCategories = new String[ExpenseIT.categoriesList.getSize()];
	 for(int i = 0; i < ExpenseIT.categoriesList.getSize(); ++i) {
		 Category cat = (Category) ExpenseIT.categoriesList.get(i);
		 allCategories[i] = cat.getName();
	 }
 }
 private void saveAndGoHome() {
	 ExpenseIT.expenseList.addExpense(ExpenseIT.expenseList, 
			 						  getInputLocation(), 
			 						  date, 
			 						  category, 
			 						  getInputAmount(), 
			 						  getInputNotes());
	 ExpenseList.saveList(ExpenseIT.expenseList);
	 jbDone.doClick();
	 clearFields();
 }
 private String getInputLocation() {
	 return jplSplash.getInLocation();
 }
 private String getInputNotes() {
	 return jplEditor.getNotes();
 }
 private String getInputAmount() {
	 //Cut string to the nearest penny
	 double value = Double.parseDouble(jplEditor.calc.getDisplayString());
	 value = (double)Math.round(value*100)/100;
	 String value2 = ("" + value);
	 if(value2.indexOf(".") == (value2.length() - 3))
		 return (value2);
	 if(value2.indexOf(".") == (value2.length() - 2))
		 return (value2 + "0");
	 else return (value2 + ".00");
 }
 private void clearFields() {
	 jplSplash.jtxLocation.setText("Where are you?");
	 jplEditor.calc.setDisplayString("0");
	 jplEditor.jtxNotes.setText("Notes:");
 }
}

@SuppressWarnings("serial")
class AddSplashPane extends JPanel {
 JLabel banner, jlbLocation;
 JTextField jtxLocation;
 JButton jbNext, jbBack;
 Dimension textFieldSize = new Dimension(1000, 30);
 
 AddSplashPane() {
	super();
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
	banner = new JLabel("Add a New Expense");
	banner.setAlignmentX(LEFT_ALIGNMENT);
	jlbLocation = new JLabel("Location:");
	jlbLocation.setAlignmentX(LEFT_ALIGNMENT);
	
	jtxLocation = new JTextField("Where are you?", 25);
	jtxLocation.setMaximumSize(textFieldSize);
	jtxLocation.setAlignmentX(LEFT_ALIGNMENT);
	
	JPanel buttonHolder = new JPanel();
	jbNext = new JButton("Next");
	jbBack = new JButton("Back");
	buttonHolder.add(jbNext);
	buttonHolder.add(jbBack);
	buttonHolder.setAlignmentX(LEFT_ALIGNMENT);
	
	add(banner);
	add(Box.createVerticalStrut(100));
	add(jlbLocation);
	add(jtxLocation);
	add(Box.createVerticalStrut(10));
	add(buttonHolder);
 }
 
 String getInLocation() {
	 return jtxLocation.getText();
 }
}

@SuppressWarnings("serial")
class CalculatorPane extends JPanel {
 JLabel banner;
 JPanel jplButtonHolder, jplMain;
 JButton[] jbCategories = new JButton[5];
 Calculator calc;
 JTextArea jtxNotes;
 String[] defaultCatsShort, defaultCatsLong;
 private CategoriesList categoryList;
 
 CalculatorPane() {
	super();
	setLayout(new BorderLayout());

	categoryList = ExpenseIT.categoriesList;
	defaultCatsShort = new String[4];
	defaultCatsLong = new String[4];

	for(int i = 0; i < categoryList.getSize(); ++i) {
		Category cat = (Category) categoryList.get(i);
		if(cat.getIsDefault()) {							//If its a default cat add it
			defaultCatsLong[i] = cat.getName();
			defaultCatsShort[i] = cat.getShortName();
		}
	} 
		
	banner = new JLabel("ExpenseIT");
	
	//Initialize the Category Buttons
	for(int i = 0; i < jbCategories.length - 1; ++i) {
		jbCategories[i] = new JButton(defaultCatsShort[i]);
		jbCategories[i].setActionCommand(defaultCatsLong[i]);
	}
	jbCategories[jbCategories.length - 1] = new JButton("Other...");

	jbCategories[0].setForeground(Color.blue);
	jbCategories[1].setForeground(Color.red);
	jbCategories[2].setForeground(Color.green);
	jbCategories[3].setForeground(Color.orange);
	
	//Set up the Categories Button Pane
	jplButtonHolder = new JPanel();
	jplButtonHolder.setLayout(new GridLayout(5, 1));
	for(int i = 0; i < jbCategories.length; ++i){
		jplButtonHolder.add(jbCategories[i]);
	}

	//Set up the Calculator Pane
	jplMain = new JPanel();
	
	JPanel calcHolder = new JPanel();
	calc = new Calculator();
	calcHolder.setLayout(new BorderLayout());
	calcHolder.add(calc, BorderLayout.PAGE_START);
	
	JPanel notesHolder = new JPanel();
	notesHolder.setLayout(new BoxLayout(notesHolder, BoxLayout.PAGE_AXIS));
	jtxNotes = new JTextArea("Notes:", 11, 20);
	jtxNotes.setLineWrap(true);
	jtxNotes.setWrapStyleWord(true);
	JScrollPane scrollPane = new JScrollPane(jtxNotes);
	scrollPane.setAlignmentX(LEFT_ALIGNMENT);
	JLabel jlNotes = new JLabel("Notes:");
	jlNotes.setAlignmentX(LEFT_ALIGNMENT);
	notesHolder.add(jlNotes);
	notesHolder.add(scrollPane);
	
	calcHolder.add(notesHolder, BorderLayout.CENTER);

	
	//Add each component
	add(banner, BorderLayout.PAGE_START);
	add(calcHolder, BorderLayout.CENTER);
	add(jplButtonHolder, BorderLayout.LINE_END);	
 }
 
 void createCategories(CategoriesList inList) {
	 
 }
 String getNotes() {
	 return jtxNotes.getText();
 }
}