package com.garryburns.ExpenseIT;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

@SuppressWarnings("serial")
class EditCatsPane extends JPanel implements ActionListener {
 private static JButton jbAdd, jbDelete, jbUndo, jbDefaultCats;
 static JButton jbBack;
 private static int indexSelected, indexDeleted;
 private static Category deletedCat;
 MyTableModel tableData;
 JTable table;
 
 EditCatsPane() {
	super();
	
	jbAdd = new JButton("Add");
	jbDelete = new JButton("Delete");
	jbUndo = new JButton("Undo");
	jbUndo.setEnabled(false);
	jbBack = new JButton("Back");
	jbDefaultCats = new JButton("Default List");
	
	jbAdd.addActionListener(this);
	jbDelete.addActionListener(this);
	jbUndo.addActionListener(this);
	jbDefaultCats.addActionListener(this);
    
	tableData = new MyTableModel();
	table = new JTable();
	table.setModel(tableData);
	table.setPreferredScrollableViewportSize(new Dimension(300, 300));
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	table.setFillsViewportHeight(true);
	JScrollPane scroller = new JScrollPane(table);
	
	add(scroller);
	add(jbAdd);
	add(jbDelete);
	add(jbUndo);
	add(jbBack);
	add(jbDefaultCats);
 }

 @Override
 public void actionPerformed(ActionEvent evt) {
	JButton jbSource = (JButton) evt.getSource();
	
	if(jbSource == jbAdd) {
		addCat();
	}
	if(jbSource == jbDelete) {
		deleteCat();
		enableDelete();
	}
	if(jbSource == jbUndo) {
		undoDelete();
	}
	if(jbSource == jbDefaultCats) {
		resetCategories();
	}
 }
 private void resetCategories() {
	ExpenseIT.categoriesList = new CategoriesList();
	tableData = new MyTableModel();
	table.setModel(tableData);
	table.repaint();
	enableDelete();
	jbUndo.setEnabled(false);
	repaint();
}

 private void addCat() {
	 String name = JOptionPane.showInputDialog(getParent(), "Enter a new Category");
	 if ((name != null) && (name.length() > 0)) {
		 ExpenseIT.categoriesList.addCategory(name);
		 table.repaint();
	 }
 }
   
   
private void deleteCat() {
	 //Delete the selected category then show the new list
	 indexSelected = table.getSelectedRow();
	 
	 //Remove Expense and store in case of undo
	 deletedCat = (Category) ExpenseIT.categoriesList.remove(indexSelected);
	 indexDeleted = indexSelected;
	 CategoriesList.saveList(ExpenseIT.categoriesList);

     setRow(indexSelected);
     table.repaint();
	
	 jbUndo.setEnabled(true);
	 repaint();
 }
 private void setRow(int row) {
	if(row > 0 && row < tableData.getRowCount())
		table.setRowSelectionInterval(row, row);
	else 
		table.setRowSelectionInterval(0, 0);
	
 }

 private void enableDelete() {
	if(tableData.getRowCount() < 1)				//Disable Delete if there's nothing to delete
		jbDelete.setEnabled(false);
	else
		jbDelete.setEnabled(true);
 }
 private void undoDelete() {
	 jbUndo.setEnabled(false);
	 ExpenseIT.categoriesList.add(indexDeleted, deletedCat);
	 CategoriesList.saveList(ExpenseIT.categoriesList);
	 
	 setRow(indexDeleted);
	 table.repaint();
	 enableDelete();
	 repaint();
 }
}

@SuppressWarnings("serial")
class MyTableModel extends AbstractTableModel {
 private String[] columnNames = {"Category Name", "Default"};
 Vector<Category> data;
 
 MyTableModel() {
	 data = ExpenseIT.categoriesList;
 }

	@Override
	public int getColumnCount() {
		return columnNames.length;
	}

	@Override
	public int getRowCount() {
		return data==null ? 0 : data.size();
	}

	@Override
	public String getColumnName(int col) {
		return columnNames[col];
	}
	@Override
	public Object getValueAt(int row, int col) {
	    if (row < 0 || row>=getRowCount())
	        return "";
	      Category cat = (Category) data.get(row);
	      switch (col) {
	        case 0: return cat.getName();
	        case 1: return cat.getIsDefault();
	      }
	      return "";
	}
	@Override
	public Class<?> getColumnClass(int c) {
		return getValueAt(0, c).getClass();
	}
	@Override
	public boolean isCellEditable(int row, int col) {
		return true;
	}
	@Override
	 public void setValueAt(Object value, int nRow, int nCol) {
	    if (nRow < 0 || nRow>=getRowCount())
	      return;
	    Category row = (Category) ExpenseIT.categoriesList.get(nRow); // (Category)data.get(nRow);

	    switch (nCol) {
	      case 0: 
	    	  row.setName(value.toString());
	    	  CategoriesList.saveList(ExpenseIT.categoriesList);
	    	break;
	      case 1:
	    	  int defaultCount = 0;
	    	  for(int i = 0; i < ExpenseIT.categoriesList.size(); ++i)
	    		  if(((Category)ExpenseIT.categoriesList.get(i)).getIsDefault())
	    			  defaultCount++;
	    	  if((Boolean) value && defaultCount < 4) {
		          row.setIsDefault((Boolean) value);
			      CategoriesList.saveList(ExpenseIT.categoriesList);
			      break;
	    	  }
	    	  else if(!((Boolean) value)){
		          row.setIsDefault((Boolean) value);
			      CategoriesList.saveList(ExpenseIT.categoriesList);
			      break;
	    	  }
	    		  
	    	  else {
	    		  JOptionPane.showMessageDialog(null, "Please only select four default Categories");
	    		  break;
	    	  }	
	    }
	  }
}