/**
 * Date: 3/03/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Parent File Names: N/A
 * Subfile Names: 
 * 			AddExpense.java
 * 			Calculator.java
 * 			ExpenseList.java
 * 			CategoriesList.java
 * 			DateO.java
 * 
 * Class Version: 0.3
 * Change Log:
 * 03/11/2011	Added methods to verify Program version - Lite, Personal, Pro
 * 				(only hard wired methods until network communication protocols are setup)
 * 				Lite - Personal Edition but limited to entering only 5 expenses at once
 * 					   and cannot add categories 
 * 					   (will include ads in the future)
 * 				Personal - No imposed limits, ad free, no User, Company, Synch options
 * 				Pro - Full implementation
 * 03/09/2011	Implemented CategoriesList class - CategoriesList.java
 * 				Added data control methods to verify proper expense inputs
 * 				Created DateO class to handle date handling/verification
 * 03/08/2011	Added Undo Button - jbUndo; Added labels to edit page
 * 				Made changes to AddExpense.java, ExpenseList.java
 * 03/07/2011	Consolidated EditPane and Home Frame into this class
 * 03/03/2011	Class Created
 * 
 * 
 * Notes: 
 * This is the main class for the program ExpenseIT
 * it makes the expense list pane the home page of the whole program.
 * 
 * Things TODO:
 * 1. Clean up UI
 * 2. Add some testing controls to the loadList() method to ensure the 
 * 		file loaded is acceptable
 * 3. Add an about page - probably put this in the Settings routine
 * 4. Add settings control and functions
 * 5. Add Sorting and possibly filters to the list pane
 */
package com.garryburns.ExpenseIT;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.sql.Date;

import javax.swing.*;

@SuppressWarnings("serial")
class ExpenseIT extends JFrame implements ActionListener {
 static ExpenseList expenseList;
 static CategoriesList categoriesList;
 static Settings settings;
 static String versionCode = "personal";
 private static JPanel jplSuper, jplMain;
 private static EditPane jplEditor;
 private static AddExpense addExpense;
 private static SettingsPane settingsPane;
 private static JButton jbAdd, jbEdit, jbDelete, jbUndo, jbSettings;
 private static JList mainList;
 private static DefaultListModel titles;
 private static int indexSelected = 1;
 private static Expense deletedExpense;
 private static int indexDeleted;
 private static final JLabel version = new JLabel("Version 0.3");
 private static String versionTitle = "ExpenseIT";
 
 private static JButton resetList;
 //TODO delete resetList and it's references - it's used only for testing purposes
 
 ExpenseIT() {
	// set up the Frame 
	super(versionTitle);
	setSize(320, 480);
	setLocation(400, 250);
	setResizable(false);
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
	loadSettings();
	checkVersion();
	setTitle(versionTitle);
	
	//Set Icon
	Image icon = Toolkit.getDefaultToolkit().getImage("icon.gif");
	setIconImage(icon);
	
	//set up the expense list
	loadExpenseList();
	loadCategoriesList();

	jplEditor = new EditPane();
	addExpense = new AddExpense();
	settingsPane = new SettingsPane();
	Font f = new Font(Font.MONOSPACED, Font.BOLD, 11);
 	//set up the list UI
	titles = new DefaultListModel();
	//titles.addElement(new Object());//Add an element to override null pointer errors from starting in Firefox
	setTitles();
	mainList = new JList(titles);
	mainList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	mainList.setLayoutOrientation(JList.VERTICAL);
	mainList.setSelectedIndex(0);
	mainList.setFont(f);
	
	//set up the main Pane to hold the list and the buttons
	jplMain = new JPanel();
	JLabel jlbHeaders;
	String sortHeaders = "Date  ::  Category  ::  Location  ::  Amount";
	
	JScrollPane listScroller = new JScrollPane(mainList);
	listScroller.setPreferredSize(new Dimension(310, 300));
	
	jlbHeaders = new JLabel(sortHeaders);
	jbAdd = new JButton("Add");
	jbEdit = new JButton("Edit");
	jbDelete = new JButton("Delete");	
	enableDeleteButton();
	jbUndo = new JButton("Undo");
	jbUndo.setEnabled(false);					//Disable Undo until there is a deleted expense
	jbSettings = new JButton("Settings");
	resetList = new JButton("Default List");
	
	jplMain.add(jlbHeaders);
	jplMain.add(listScroller);
	jplMain.add(jbAdd);
	jplMain.add(jbEdit);
	jplMain.add(jbDelete);
	jplMain.add(jbUndo);
	jplMain.add(jbSettings);
	jplMain.add(resetList);
	//End Main Pane Setup
	
	//Setup the Version Label Pane
	JPanel versionPane = new JPanel();
	versionPane.add(version);

	
	// add ActionListeners
	jbAdd.addActionListener(this);
	jbEdit.addActionListener(this);
	jbDelete.addActionListener(this);
	jbUndo.addActionListener(this);
	jbSettings.addActionListener(this);
	jplEditor.jbSave.addActionListener(this);
	jplEditor.jbCancel.addActionListener(this);
	addExpense.jbDone.addActionListener(this);
	addExpense.jbHome.addActionListener(this);
	settingsPane.jbExitSettings.addActionListener(this);
	resetList.addActionListener(this);
	
	jplSuper = new JPanel();
	jplSuper.setLayout(new BorderLayout());
	jplSuper.add(jplMain, BorderLayout.CENTER);
	jplSuper.add(versionPane, BorderLayout.PAGE_END);
	setContentPane(jplSuper);
	setVisible(true);	
 }  // End of Constructor
 

 @Override
 public void actionPerformed(ActionEvent evt) {
	 JButton jbSource = (JButton) evt.getSource();
	 
	 if(jbSource == resetList) {
		 expenseList = new ExpenseList();
		 enableDeleteButton();
		 jbUndo.setEnabled(false);
		 setTitles();
		 repaint();
	 }
	 if(jbSource == addExpense.jbHome) {
		 goHome();
	 }
	 if(jbSource == addExpense.jbDone) {
		 //Come home
		 indexSelected = expenseList.getSize() - 1;
		 goHome();
	 }
	 if(jbSource == settingsPane.jbExitSettings) {
		 goHome();
	 }
	 if(jbSource == jbAdd){
		 // cannot simply make a new instance of AddExpense because the action listeners
		 //	were set up for the old one and won't transfer to the new instance
		 if(isLite()) {
			 return; 
		 }
		 else {
			 addExpense.jplEditor.setDefaultButtons();
			 AddExpense.loadCatNames();
			 setContentPane(addExpense);
			 setVisible(true);
			 repaint();
		 }
	 }
	 if(jbSource == jbEdit) {
		 //Move to the editor screen
		 if(mainList.getSelectedIndex() == -1) 		//Make sure an existing expense was selected
			 JOptionPane.showMessageDialog(getParent(), "Please choose an expense to edit.");
		 else{
			 indexSelected = mainList.getSelectedIndex();
			 populateEditor(indexSelected);
			 setContentPane(jplEditor);
			 setVisible(true);
			 repaint();
		 }
	 }
	 if(jbSource == jbDelete) {
		 deleteExpense();
		 enableDeleteButton();
	 }
	 if(jbSource == jbUndo){
		 undoDelete();
	 }
	 if(jbSource == jbSettings) {
		//Move to Settings Pane
		 setContentPane(settingsPane);
		 setVisible(true);
		 repaint();
	 }
	 if(jbSource == jplEditor.jbCancel) {
		 //Return Home from the Editor with out saving changes
		 goHome();
	 }
	 if(jbSource == jplEditor.jbSave) {
		 //Save the changes made in the Editor and return Home
		 if(verifyData()) {
		 setExpense(indexSelected);
		 goHome();
		 }
	 }	
 }
 private boolean isLite() {
	if(versionCode.contains("lite") && expenseList.getSize() >= 5) {
		JOptionPane.showMessageDialog(getParent(), 
								"You have the maximum allowed expenses"
								+ "\n Please upgrade your version in Settings to add more",
								"ExpenseIT Lite",
								JOptionPane.WARNING_MESSAGE);
		return true;
	}
	else return false;
}


private void loadSettings() {
	 File file = new File("ExpenseIT.dat");
	 // if there's a list already saved in ExpenseIT.exp - load that list
	 if(file.exists()) {
		 try {
			 FileInputStream fi = new FileInputStream("ExpenseIT.dat");
			 ObjectInputStream oi = new ObjectInputStream(fi);
			 settings = (Settings) oi.readObject();
			 // TODO add some testing to ensure list is viable
			 oi.close();
		 } catch (Exception e) {
			 System.out.println("Error - " + e.toString());
		 }
	 } 
	 // if no previous list exists (there's no file) then setup a new default list
	 else { 
		 settings = new Settings();
	 }
}
 private void loadCategoriesList() {
	 File file = new File("ExpenseIT.cats");
	 // if there's a category list already saved in ExpenseIT.cat - load that list
	 if(file.exists()) {
		 try {
			 FileInputStream fi = new FileInputStream("ExpenseIT.cats");
			 ObjectInputStream oi = new ObjectInputStream(fi);
			 categoriesList = (CategoriesList) oi.readObject();
			 // TODO add some testing to ensure list is viable
			 oi.close();
		 } catch (Exception e) {
			 System.out.println("Error - " + e.toString());
		 }
	 } 
	 // if no previous list exists (there's no file) then setup a new default list
	 else { 
		 categoriesList = new CategoriesList();
	 }
 }
 private void loadExpenseList() {
	 File file = new File("ExpenseIT.exp");
	 // if there's a list already saved in ExpenseIT.exp - load that list
	 if(file.exists()) {
		 try {
			 FileInputStream fi = new FileInputStream("ExpenseIT.exp");
			 ObjectInputStream oi = new ObjectInputStream(fi);
			 expenseList = (ExpenseList) oi.readObject();
			 // TODO add some testing to ensure list is viable
			 oi.close();
		 } catch (Exception e) {
			 System.out.println("Error - " + e.toString());
		 }
	 } 
	 // if no previous list exists (there's no file) then setup a new default list
	 else { 
		 expenseList = new ExpenseList();
	 }
 }
 private void enableDeleteButton() {
	 if(expenseList.getSize() < 1)				//Disable Delete if there's nothing to delete
		jbDelete.setEnabled(false);
	 else {
		 jbDelete.setEnabled(true);
	 }
 }
 private void deleteExpense() {
	 //Delete the selected expense then show the new list
	 indexSelected = mainList.getSelectedIndex();
	 deletedExpense = (Expense) expenseList.remove(indexSelected);	//Remove Expense and store in case of undo
	 indexDeleted = indexSelected;				//Save the deleted expense's index
	 ExpenseList.saveList(expenseList);
	 jbUndo.setEnabled(true);
	 setTitles();
	 if(indexSelected < expenseList.getSize())
		 mainList.setSelectedIndex(indexSelected);
	 else mainList.setSelectedIndex(indexSelected - 1);
	 repaint();
 }
 private void undoDelete() {
	 jbUndo.setEnabled(false);
	 expenseList.add(indexDeleted, deletedExpense);
	 ExpenseList.saveList(expenseList);
	 setTitles();
	 mainList.setSelectedIndex(indexDeleted);
	 enableDeleteButton();
	 repaint();
 }
 private boolean verifyData() {
	 //TODO verify category is part of the cat list or confirm to add it
	 String catName = jplEditor.jtxCat.getText();
	 boolean partOfList = false;
	 for(int i = 0; i < categoriesList.size(); ++i) {
		 Category cat = (Category) categoriesList.get(i);
		 if(catName.equals(cat.getName())){
			 partOfList = true;
		 }
	 }
	 if(!partOfList) {
		 if(!versionCode.contains("lite")) {
			 Category cat = new Category(catName);
			 categoriesList.addElement(cat);
			 CategoriesList.saveList(categoriesList);
			 JOptionPane.showMessageDialog(getParent(), cat.getName() + " added to categories");
		 }
		 else{
			 jplEditor.jtxCat.setText(((Expense) expenseList.elementAt(indexSelected)).getCategory());
			 JOptionPane.showMessageDialog(getParent(), "Please upgrade your version in Settings"
					 									+ " to edit available categories",
					 									"ExpenseIT Lite",
					 									JOptionPane.WARNING_MESSAGE);
		 }
	 }
	 //Date is verified using the DateO class
	 //Only verify location is not empty
	 if(jplEditor.jtxLoc.getText().isEmpty()){
		 String text = (String) JOptionPane.showInputDialog(getParent(), "Please enter a Location:");
		 jplEditor.jtxLoc.setText(text);
	 }
	 //Verify only numbers were entered and round to the nearest penny if necessary
	 try {
		 double value = Double.parseDouble(jplEditor.jtxAmt.getText());
		 value = (double)Math.round(value*100)/100;
		 String value2 = ("" + value);
		 if(value2.indexOf(".") == (value2.length() - 3))
			 jplEditor.jtxAmt.setText(value2);
		 if(value2.indexOf(".") == (value2.length() - 2))
			 jplEditor.jtxAmt.setText(value2 + "0");
		 return true;
	 } catch (Exception e) {
		 JOptionPane.showMessageDialog(getParent(), "Please enter only numbers for Amount");
		 return false;
	 }	 
 }
 
 private void setExpense(int index) {
	 Date dateHolder = DateO.makeDate(jplEditor.jtxDate.getText());
	 String dateStringHolder = DateO.makeString(dateHolder);
	 Expense exp = new Expense(dateStringHolder, 
			 				   jplEditor.jtxCat.getText(), 
			 				   jplEditor.jtxLoc.getText(), 
			 				   jplEditor.jtxAmt.getText(), 
			 				   jplEditor.jtxNotes.getText());
	 expenseList.set(indexSelected, exp);
	 ExpenseList.saveList(expenseList);
 }
 private void populateEditor(int index) {
	 Expense editedExpense = (Expense) expenseList.get(index);
	 jplEditor.jtxCat.setText(editedExpense.getCategory());
	 jplEditor.jtxDate.setText(editedExpense.getDate());
	 jplEditor.jtxLoc.setText(editedExpense.getLocation());
	 jplEditor.jtxAmt.setText(editedExpense.getAmount());
	 jplEditor.jtxNotes.setText(editedExpense.getNotes());
 }
 private void setTitles() {
	 titles.clear();
	 for(int i = 0; i < expenseList.size(); ++i){
		 Expense exp = (Expense) expenseList.elementAt(i);
		 //Normalize names before putting them in the title
		 String normCat = normalizeString(exp.getCategory());
		 String normLoc = normalizeString(exp.getLocation());
		 String title = (exp.getDate() + " : " + normCat + " : " 
				 + normLoc + " : " + exp.getAmount());
		 titles.addElement(title);
	 }
 }
 private String normalizeString(String inString) {
	String norm = inString;
	if(inString.length() < 9) {
		norm = (inString);
		while(norm.length() < 9){
			norm = (norm + " ");
		}
	}	
	if(inString.length() > 9){
		norm = (inString.substring(0, 9));
	}
	return norm;
}

 private void checkVersion() {
 	// TODO verify authentication code with server (verify license is up to date and which version code)
	 if(settings.getAuthenticationCode().contains("personal")){
		 versionCode = "personal"; 
		 versionTitle = "ExpenseIT Personal";
		 return;
	 }
	 else if(settings.getAuthenticationCode().contains("professional")){
		 versionCode = "professional";
		 versionTitle = "ExpenseIT Pro";
		 return;
	 }
	 else { 
		 versionCode = "lite";
		 versionTitle = "ExpenseIT Lite";
		 return;
	 }
 }


public void goHome() {
	 jbUndo.setEnabled(false);
	 setTitles();
	 mainList.setSelectedIndex(indexSelected);
	 enableDeleteButton();
	 setContentPane(jplSuper);
	 setVisible(true);
	 repaint();
	 return;
 }
 public static void main(String[] args) {
	new ExpenseIT();
 }
}


@SuppressWarnings("serial")
class EditPane extends JPanel {
 JButton jbSave, jbCancel;
 JLabel jlCat, jlDate, jlLoc, jlAmt, jlNotes;
 JTextField jtxCat, jtxDate, jtxLoc, jtxAmt;
 JTextArea jtxNotes;
 
 EditPane() {
	 super();
	 setLayout(new BorderLayout());
	 
	 JPanel buttonHolder = new JPanel();
	 buttonHolder.setLayout(new BoxLayout(buttonHolder, BoxLayout.LINE_AXIS));
	 jbSave = new JButton("Save");
	 jbCancel = new JButton("Cancel");
	 buttonHolder.add(Box.createHorizontalGlue());
	 buttonHolder.add(jbCancel);
	 buttonHolder.add(jbSave);
	 
	 JPanel editHolder = new JPanel();
	 editHolder.setLayout(new BoxLayout(editHolder, BoxLayout.PAGE_AXIS));
	 jtxCat = new JTextField("Category", 20);
	 jtxCat.setAlignmentX(LEFT_ALIGNMENT);
	 jtxDate = new JTextField("Date", 20);
	 jtxDate.setAlignmentX(LEFT_ALIGNMENT);
	 jtxLoc = new JTextField("Location", 20);
	 jtxLoc.setAlignmentX(LEFT_ALIGNMENT);
	 jtxAmt = new JTextField("$Amount", 20);
	 jtxAmt.setAlignmentX(LEFT_ALIGNMENT);
	 jtxNotes = new JTextArea("Notes:", 13, 20);
	 jtxNotes.setLineWrap(true);
	 jtxNotes.setWrapStyleWord(true);
	 JScrollPane scrollPane = new JScrollPane(jtxNotes);
	 scrollPane.setAlignmentX(LEFT_ALIGNMENT);
	 
	 jlCat = new JLabel("Category:");
	 jlCat.setAlignmentX(LEFT_ALIGNMENT);
	 jlDate = new JLabel("Date:");
	 jlDate.setAlignmentX(LEFT_ALIGNMENT);
	 jlLoc = new JLabel("Location:");
	 jlLoc.setAlignmentX(LEFT_ALIGNMENT);
	 jlAmt = new JLabel("Amount:");
	 jlAmt.setAlignmentX(LEFT_ALIGNMENT);
	 jlNotes = new JLabel("Notes:");
	 jlNotes.setAlignmentX(LEFT_ALIGNMENT);
	 
	 
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlCat);
	 editHolder.add(jtxCat);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlDate);
	 editHolder.add(jtxDate);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlLoc);
	 editHolder.add(jtxLoc);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlAmt);
	 editHolder.add(jtxAmt);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlNotes);
	 editHolder.add(scrollPane);
	 
	 add(buttonHolder, BorderLayout.PAGE_START);
	 add(editHolder, BorderLayout.CENTER);
	 setVisible(true);
 }
}