/**
 * Date: 3/03/11
 * Programmer: Garry Burns
 * Project Name: ExpenseIT
 * Parent File Names: N/A
 * Subfile Names: 
 * 			AddExpense.java
 * 			Calculator.java
 * 			ExpenseList.java
 * 			CategoriesList.java
 * 			DateO.java
 * 
 * Class Version: 0.5
 * Change Log:
 * 03/17/2011	Completed migration of categories into Table format
 * 				Added methods to set user/company credentials
 * 03/11/2011	Added methods to verify Program version - Lite, Personal, Pro
 * 				(only hard wired methods until network communication protocols are setup)
 * 				Lite - Personal Edition but limited to entering only 5 expenses at once
 * 					   and cannot add categories 
 * 					   (will include ads in the future)
 * 				Personal - No imposed limits, ad free, no User, Company, Synch options
 * 				Pro - Full implementation
 * 03/09/2011	Implemented CategoriesList class - CategoriesList.java
 * 				Added data control methods to verify proper expense inputs
 * 				Created DateO class to handle date handling/verification
 * 03/08/2011	Added Undo Button - jbUndo; Added labels to edit page
 * 				Made changes to AddExpense.java, ExpenseList.java
 * 03/07/2011	Consolidated EditPane and Home Frame into this class
 * 03/03/2011	Class Created
 * 
 * 
 * Notes: 
 * This is the main class for the program ExpenseIT
 * it makes the expense list pane the home page of the whole program.
 * 
 * Things TODO:
 * 1. Clean up UI
 * 2. Add some testing controls to the loadList() method to ensure the 
 * 		file loaded is acceptable
 * 3. Add an about page - probably put this in the Settings routine
 * 4. Add settings control and functions
 * 5. Add Sorting and possibly filters to the list pane
 */
package com.garryburns.ExpenseIT;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.Date;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.TableColumn;

@SuppressWarnings("serial")
class ExpenseIT extends JFrame implements ActionListener {
 static ExpenseTableModel expenseList;
 static CategoriesList categoriesList;
 static Settings settings;
 static String versionCode = "personal";
 static String aboutMe = "ExpenseIT Ver. 0.5\n\u00a9 Garry Burns\nwww.garryburns.com/java";
 static Object[] savedInfo = {settings, expenseList, categoriesList};
 private static JPanel jplSuper, jplMain;
 private static EditPane jplEditor;
 private static AddExpense addExpense;
 private static SettingsPane settingsPane;
 private static JButton jbAdd, jbEdit, jbDelete, jbUndo, jbSettings;
 private static JTable table;
 private static int indexSelected = 1;
 private static Expense deletedExpense;
 private static int indexDeleted;
 private static final JLabel version = new JLabel("Version 0.5");
 private static String versionTitle = "ExpenseIT";
 
 private static JButton resetList;
 //TODO delete resetList and it's references - it's used only for testing purposes
 
 ExpenseIT() {
	// set up the Frame 
	super(versionTitle);
	setSize(320, 480);
	setLocation(400, 250);
	setResizable(false);
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
	showTesterInfo();
	
	loadSavedInfo();
	checkVersion();
	setTitle(versionTitle);
	
	//Set Icon
	Image icon = Toolkit.getDefaultToolkit().getImage("icon.gif");
	setIconImage(icon);

	jplEditor = new EditPane();
	addExpense = new AddExpense();
	settingsPane = new SettingsPane(this);
 	//set up the list UI	
	expenseList = new ExpenseTableModel();
	table = new JTable(expenseList);
	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	table.setFillsViewportHeight(true);
	for (int i = 0; i < expenseList.getColumnCount(); ++i) {
		TableColumn col = table.getColumnModel().getColumn(i);
		if (i == 0)
			col.setPreferredWidth(60);
		else if (i == 3)
			col.setPreferredWidth(50);
		else
			col.setPreferredWidth(80);
	}
	
	//set up the main Pane to hold the list and the buttons
	jplMain = new JPanel();
	
	JScrollPane listScroller = new JScrollPane(table);
	listScroller.setPreferredSize(new Dimension(310, 300));
	
	jbAdd = new JButton("Add");
	jbEdit = new JButton("Edit");
	jbDelete = new JButton("Delete");	
	enableDeleteButton();
	jbUndo = new JButton("Undo");
	jbUndo.setEnabled(false);					//Disable Undo until there is a deleted expense
	jbSettings = new JButton("Settings");
	resetList = new JButton("Default List");
	resetList.setToolTipText("Tester's Tool: Resets the Expense List to default values");
	resetList.setBackground(Color.green);
	
	JPanel testerp = new JPanel();
	testerp.add(Box.createHorizontalStrut(200));
	testerp.add(resetList);
	testerp.add(Box.createHorizontalStrut(200));
	
	jplMain.add(listScroller);
	jplMain.add(jbAdd);
	jplMain.add(jbEdit);
	jplMain.add(jbDelete);
	jplMain.add(jbUndo);
	jplMain.add(jbSettings);
	jplMain.add(testerp);
	//End Main Pane Setup
	
	//Setup the Version Label Pane
	JPanel versionPane = new JPanel();
	versionPane.add(version);

	
	// add ActionListeners
	jbAdd.addActionListener(this);
	jbEdit.addActionListener(this);
	jbDelete.addActionListener(this);
	jbUndo.addActionListener(this);
	jbSettings.addActionListener(this);
	jplEditor.jbSave.addActionListener(this);
	jplEditor.jbCancel.addActionListener(this);
	addExpense.jbDone.addActionListener(this);
	addExpense.jbHome.addActionListener(this);
	settingsPane.jbExitSettings.addActionListener(this);
	resetList.addActionListener(this);
	
	jplSuper = new JPanel();
	jplSuper.setLayout(new BorderLayout());
	jplSuper.add(jplMain, BorderLayout.CENTER);
	jplSuper.add(versionPane, BorderLayout.PAGE_END);
	setContentPane(jplSuper);
	setVisible(true);	
 }  // End of Constructor
 

 private void showTesterInfo() {
	 JOptionPane.showMessageDialog(getParent(), 
			 "Welcome to the Pre-Release, Version 0.5, of ExpenseIT.\nMost functions are fully operational."
			 + " (Skin chooser and internet synching are the only inoperable functions)."
			 + "\nA few tester tools have been added for convience, they appear as green buttons or italic text"
			 + "\nPlease report any bugs, comments, or suggestions to garrybur@gmail.com");
}


@Override
 public void actionPerformed(ActionEvent evt) {
	 JButton jbSource = (JButton) evt.getSource();
	 
	 if(jbSource == resetList) {
		 expenseList = new ExpenseTableModel();
		 enableDeleteButton();
		 jbUndo.setEnabled(false);
		 repaint();
	 }
	 if(jbSource == addExpense.jbHome) {
		 goHome();
	 }
	 if(jbSource == addExpense.jbDone) {
		 //Come home
		 indexSelected = expenseList.getRowCount() - 1;
		 goHome();
	 }
	 if(jbSource == settingsPane.jbExitSettings) {
		 goHome();
	 }
	 if(jbSource == jbAdd){
		 // cannot simply make a new instance of AddExpense because the action listeners
		 //	were set up for the old one and won't transfer to the new instance
		 if(isLite()) {
			 return; 
		 }
		 else {
			 addExpense.jplEditor.setDefaultButtons();
			 AddExpense.loadCatNames();
			 setContentPane(addExpense);
			 setVisible(true);
			 repaint();
		 }
	 }
	 if(jbSource == jbEdit) {
		 //Move to the editor screen
		 if(table.getSelectedRow() == -1) 		//Make sure an existing expense was selected
			 JOptionPane.showMessageDialog(getParent(), "Please choose an expense to edit.");
		 else{
			 indexSelected = table.getSelectedRow();
			 populateEditor(indexSelected);
			 setContentPane(jplEditor);
			 setVisible(true);
			 repaint();
		 }
	 }
	 if(jbSource == jbDelete) {
		 deleteExpense();
		 enableDeleteButton();
	 }
	 if(jbSource == jbUndo){
		 undoDelete();
	 }
	 if(jbSource == jbSettings) {
		//Move to Settings Pane
		 setContentPane(settingsPane);
		 setVisible(true);
		 repaint();
	 }
	 if(jbSource == jplEditor.jbCancel) {
		 //Return Home from the Editor with out saving changes
		 goHome();
	 }
	 if(jbSource == jplEditor.jbSave) {
		 //Save the changes made in the Editor and return Home
		 if(verifyData()) {
		 setExpense(indexSelected);
		 goHome();
		 }
	 }	
 }
 private boolean isLite() {
	if(versionCode.contains("lite") && expenseList.getRowCount() >= 5) {
		JOptionPane.showMessageDialog(getParent(), 
								"You have the maximum allowed expenses"
								+ "\n Please upgrade your version in Settings",
								"ExpenseIT Lite",
								JOptionPane.WARNING_MESSAGE);
		return true;
	}
	else return false;
}

 
 static void loadSavedInfo() {
	 File file = new File("ExpenseIT.dat");
	 if(file.exists()) {
		 try {
			 FileInputStream fi = new FileInputStream("ExpenseIT.dat");
			 ObjectInputStream oi = new ObjectInputStream(fi);
			 savedInfo = (Object[]) oi.readObject();
			 oi.close();
		 } catch (Exception e) {
			 System.out.println("Error - " + e.toString());
			 int i = JOptionPane.showConfirmDialog(null, 
				 		"Corrupted data file.\nWould you like to rebuild the database?" +
				 		"\n(All previously saved data will be lost)",
				 		"ExpenseIT Load Error", 
				 		JOptionPane.ERROR_MESSAGE,
				 		JOptionPane.YES_NO_OPTION);
			 if(i == JOptionPane.YES_OPTION){
				 settings = new Settings();
				 expenseList = new ExpenseTableModel();
				 categoriesList = new CategoriesList();
				 savedInfo[0] = settings;
				 savedInfo[1] = expenseList;
				 savedInfo[2] = categoriesList;
				 save();
			 }
			 else System.exit(-1);
		 }
		 settings = (Settings) savedInfo[0];
		 expenseList = (ExpenseTableModel) savedInfo[1];
		 categoriesList = (CategoriesList) savedInfo[2];
	 } 
	 // if no previous list exists (there's no file) then setup a new default list
	 else { 
		 settings = new Settings();
		 expenseList = new ExpenseTableModel();
		 categoriesList = new CategoriesList();
		 savedInfo[0] = settings;
		 savedInfo[1] = expenseList;
		 savedInfo[2] = categoriesList;
		 save();
	 }
 }
 
 static void save() {
	 try {
		 FileOutputStream fo = new FileOutputStream("ExpenseIT.dat");
		 ObjectOutputStream oo = new ObjectOutputStream(fo);
		 oo.writeObject(savedInfo);
		 oo.close();
	 } catch (IOException e) {
		 System.out.println("Error - " + e.toString());
	 }

 }
 private void enableDeleteButton() {
	 if(expenseList.getRowCount() < 1)				//Disable Delete if there's nothing to delete
		jbDelete.setEnabled(false);
	 else {
		 jbDelete.setEnabled(true);
	 }
 }
 private void deleteExpense() {
	 //Delete the selected expense then show the new list
	 indexSelected = table.getSelectedRow();
	 deletedExpense = (Expense) expenseList.remove(indexSelected);
	 indexDeleted = indexSelected;
	 save();
	 jbUndo.setEnabled(true);
	 setRow(indexSelected);
	 repaint();
 }
 private void undoDelete() {
	 jbUndo.setEnabled(false);
	 expenseList.addExpense(indexDeleted, deletedExpense);
	 setRow(indexDeleted);
	 enableDeleteButton();
	 repaint();
 }
 private boolean verifyData() {
	 String catName = jplEditor.jtxCat.getText();
	 boolean partOfList = false;
	 for(int i = 0; i < categoriesList.size(); ++i) {
		 if(catName.equals(((Category) categoriesList.get(i)).getName()))
			 partOfList = true;
	 }
	 if(!partOfList) {
		 if(!versionCode.contains("lite")) {
			 Category cat = new Category(catName);
			 categoriesList.addElement(cat);
			 CategoriesList.saveList(categoriesList);
			 JOptionPane.showMessageDialog(getParent(), cat.getName() + " added to categories");
		 }
		 else{
			 jplEditor.jtxCat.setText(((Expense) expenseList.get(indexSelected)).getCategory());
			 JOptionPane.showMessageDialog(getParent(), "Please upgrade your version in Settings"
					 									+ " to edit available categories",
					 									"ExpenseIT Lite",
					 									JOptionPane.WARNING_MESSAGE);
		 }
	 }
	 //Date is verified using the DateO class
	 //Only verify location is not empty
	 if(jplEditor.jtxLoc.getText().isEmpty()){
		 String text = (String) JOptionPane.showInputDialog(getParent(), "Please enter a Location:");
		 jplEditor.jtxLoc.setText(text);
	 }
	 //Verify only numbers were entered and round to the nearest penny if necessary
	 try {
		 double value = Double.parseDouble(jplEditor.jtxAmt.getText());
		 value = (double)Math.round(value*100)/100;
		 String value2 = ("" + value);
		 if(value2.indexOf(".") == (value2.length() - 3))
			 jplEditor.jtxAmt.setText(value2);
		 if(value2.indexOf(".") == (value2.length() - 2))
			 jplEditor.jtxAmt.setText(value2 + "0");
		 return true;
	 } catch (Exception e) {
		 JOptionPane.showMessageDialog(getParent(), "Please enter only numbers for Amount");
		 return false;
	 }	 
 }
 
 private void setExpense(int index) {
	 Date dateHolder = DateO.makeDate(jplEditor.jtxDate.getText());
	 String dateStringHolder = DateO.makeString(dateHolder);
	 Expense exp = new Expense(dateStringHolder, 
			 				   jplEditor.jtxCat.getText(), 
			 				   jplEditor.jtxLoc.getText(), 
			 				   jplEditor.jtxAmt.getText(), 
			 				   jplEditor.jtxNotes.getText());
	 expenseList.set(indexSelected, exp);
 }
 private void populateEditor(int index) {
	 Expense editedExpense = (Expense) expenseList.get(index);
	 jplEditor.jtxCat.setText(editedExpense.getCategory());
	 jplEditor.jtxDate.setText(editedExpense.getDate());
	 jplEditor.jtxLoc.setText(editedExpense.getLocation());
	 jplEditor.jtxAmt.setText(editedExpense.getAmount());
	 jplEditor.jtxNotes.setText(editedExpense.getNotes());
 }

 private void setRow(int row) {
		if(row > 0 && row < expenseList.getRowCount())
			table.setRowSelectionInterval(row, row);
		else 
			table.setRowSelectionInterval(0, 0);
		
}

 private void checkVersion() {
 	// TODO verify authentication code with server (verify license is up to date and which version code)
	 if(settings.getAuthenticationCode().contains("personal")){
		 versionCode = "personal"; 
		 versionTitle = "ExpenseIT Personal";
		 return;
	 }
	 else if(settings.getAuthenticationCode().contains("professional")){
		 versionCode = "professional";
		 versionTitle = "ExpenseIT Pro";
		 return;
	 }
	 else { 
		 versionCode = "lite";
		 versionTitle = "ExpenseIT Lite";
		 return;
	 }
 }


public void goHome() {
	 jbUndo.setEnabled(false);
	 setRow(indexSelected);
	 enableDeleteButton();
	 setContentPane(jplSuper);
	 setVisible(true);
	 repaint();
	 return;
 }
 public static void main(String[] args) {
	new ExpenseIT();
 }
}


@SuppressWarnings("serial")
class EditPane extends JPanel {
 JButton jbSave, jbCancel;
 JLabel jlCat, jlDate, jlLoc, jlAmt, jlNotes;
 JTextField jtxCat, jtxDate, jtxLoc, jtxAmt;
 JTextArea jtxNotes;
 
 EditPane() {
	 super();
	 setLayout(new BorderLayout());
	 
	 JPanel buttonHolder = new JPanel();
	 buttonHolder.setLayout(new BoxLayout(buttonHolder, BoxLayout.LINE_AXIS));
	 jbSave = new JButton("Save");
	 jbCancel = new JButton("Cancel");
	 buttonHolder.add(Box.createHorizontalGlue());
	 buttonHolder.add(jbCancel);
	 buttonHolder.add(jbSave);
	 
	 JPanel editHolder = new JPanel();
	 editHolder.setLayout(new BoxLayout(editHolder, BoxLayout.PAGE_AXIS));
	 jtxCat = new JTextField("Category", 20);
	 jtxCat.setAlignmentX(LEFT_ALIGNMENT);
	 jtxDate = new JTextField("Date", 20);
	 jtxDate.setAlignmentX(LEFT_ALIGNMENT);
	 jtxLoc = new JTextField("Location", 20);
	 jtxLoc.setAlignmentX(LEFT_ALIGNMENT);
	 jtxAmt = new JTextField("$Amount", 20);
	 jtxAmt.setAlignmentX(LEFT_ALIGNMENT);
	 jtxNotes = new JTextArea("Notes:", 13, 20);
	 jtxNotes.setLineWrap(true);
	 jtxNotes.setWrapStyleWord(true);
	 JScrollPane scrollPane = new JScrollPane(jtxNotes);
	 scrollPane.setAlignmentX(LEFT_ALIGNMENT);
	 
	 jlCat = new JLabel("Category:");
	 jlCat.setAlignmentX(LEFT_ALIGNMENT);
	 jlDate = new JLabel("Date:");
	 jlDate.setAlignmentX(LEFT_ALIGNMENT);
	 jlLoc = new JLabel("Location:");
	 jlLoc.setAlignmentX(LEFT_ALIGNMENT);
	 jlAmt = new JLabel("Amount:");
	 jlAmt.setAlignmentX(LEFT_ALIGNMENT);
	 jlNotes = new JLabel("Notes:");
	 jlNotes.setAlignmentX(LEFT_ALIGNMENT);
	 
	 
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlCat);
	 editHolder.add(jtxCat);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlDate);
	 editHolder.add(jtxDate);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlLoc);
	 editHolder.add(jtxLoc);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlAmt);
	 editHolder.add(jtxAmt);
	 editHolder.add(Box.createVerticalStrut(5)); // puts a vertical spacer
	 editHolder.add(jlNotes);
	 editHolder.add(scrollPane);
	 
	 add(buttonHolder, BorderLayout.PAGE_START);
	 add(editHolder, BorderLayout.CENTER);
	 setVisible(true);
 }
}